
list = findfile("mag*.final")

nfiles = n_elements(list)

; first thing that we want to do is determine how many stations we have
; total.  This means that we have to open each file and get a list of
; stations from that file, and compare it to a list of all of the 
; previous files.

nstats_infile = intarr(nfiles)
indices       = intarr(nfiles,200)

for ifile = 0, nfiles-1 do begin

  print, list(ifile)

  openr,ifile+1, list(ifile)

  line = ''
  done = 0

  minute = 0
  nlines = 0

  while not done do begin

    readf,ifile+1,line
    minute = fix(strmid(line,9,2))
    stat = strmid(line,12,3)

    if (nlines eq 0) then begin
      start_minute = minute
      if (ifile eq 0) then statlist = [stat]
    endif

    nlines = nlines+1

    if (start_minute ne minute) then begin
      done = 1
    endif else begin

      nstats = n_elements(statlist)

      found = 0
      istat = 0
      while not found do begin
        if (statlist(istat) eq stat) then begin
          found = 1
        endif else begin
          if (istat eq nstats-1) then begin
            found = 1
            istat = -1
          endif else istat = istat + 1
        endelse
      endwhile

      if (istat eq -1) then begin
        statlist = [statlist,stat]
        indices(ifile,nlines-1) = nstats
      endif else indices(ifile,nlines-1) = istat

    endelse      

  endwhile

  nstats_infile(ifile) = nlines-1

  close, ifile+1

endfor

ntimesPerFile = 1440
ntimes = long(nfiles)*long(ntimesPerFile)

nstats = n_elements(statlist)

times = dblarr(ntimes)
data  = fltarr(nstats, 3, ntimes) - 99999.0

itime_i = intarr(6)

iTimeTotal = long(0)

for ifile = 0, nfiles-1 do begin

  print, list(ifile)
  openr,ifile+1, list(ifile)

  for itime = 0, ntimesPerFile-1 do begin

    for istat = 0, nstats_infile(ifile)-1 do begin

      readf,ifile+1,line

      if istat eq 0 then begin
        itime_i(0) = fix(strmid(line,0,2))
        itime_i(1) = fix(strmid(line,2,2))
        itime_i(2) = fix(strmid(line,4,2))
        itime_i(3) = fix(strmid(line,7,2))
        itime_i(4) = fix(strmid(line,9,2))
	itime_i(5) = 0
        c_a_to_r, itime_i, rtime
        times(iTimeTotal) = rtime
      endif

      data(indices(ifile,istat),0,iTimeTotal) = float(strmid(line, 19,6))
      data(indices(ifile,istat),1,iTimeTotal) = float(strmid(line, 25,6))
      data(indices(ifile,istat),2,iTimeTotal) = float(strmid(line, 31,6))

    endfor

    iTimeTotal = iTimeTotal + long(1)

  endfor

  close, ifile+1

endfor

setdevice,'stats.ps','l',4,0.95

ppp = 3
space = 0.01
pos_space, ppp, space, sizes, ny = ppp

get_position,ppp,space,sizes,0,pos1,/rect
get_position,ppp,space,sizes,1,pos2,/rect
get_position,ppp,space,sizes,2,pos3,/rect
pos1(0) = pos1(0) + 0.05
pos2(0) = pos2(0) + 0.05
pos3(0) = pos3(0) + 0.05

pos = fltarr(3,4)
pos(0,*) = pos1
pos(1,*) = pos2
pos(2,*) = pos3

stime = min(times)
etime = max(times)

time_axis, stime, etime, s_time_range, e_time_range,        	$
	xtickname, xtitle, xtickvalue, xminor, xtickn

comp = ['X','Y','Z']

for i=0,nstats-1 do begin

  print, statlist(i)

  plotdumb

  for icomp = 0,2 do begin

    if (icomp eq 2) then begin
      xtn = xtickname
      xt  = xtitle
    endif else begin
      xtn = strarr(20)+' '
      xt  = ' '
    endelse

    title = ' '
    if (icomp eq 0) then title = statlist(i)+' '+tostr(i+1)+' of '+tostr(nstats)

    plot, times-stime,data(i,icomp,*), pos = pos(icomp,*),	 $
	xstyle = 1, /noerase,		$
	xtickname = xtn, xtickv=xtickvalue, 			$
	xticks = xtickn, xminor = xminor, xtitle = xt,		$
	xrange = [s_time_range, e_time_range],                  $
        ytitle = comp(icomp), yrange = yrange, min_val = -5000.0, $
        title = title

    for ifile = 1, nfiles-1 do begin
       it = long(ntimesPerFile)*long(ifile)
       oplot, [times(it), times(it)]-stime,[-5000,5000], linestyle = 1
    endfor

    oplot, [s_time_range, e_time_range], [0.0,0.0], linestyle = 1

    loc = where(data(i,icomp,*) gt -5000.0,count)
    if (count gt 0) then m = median(data(i,icomp,loc)) else m = -99999.

    ms = strcompress(string(m))

    xyouts, pos(icomp,2)+0.02, pos(icomp,1), ms, orient = 90, /norm

  endfor

endfor

end
