pro read_UVI,UVI_file,duts,hal,ped,lats,mlts,oned=oned, indicate=indicate

  if keyword_set(oned) then oned = 1 else oned = 0
  if keyword_set(indicate) then indicate = 1 else indicate = 0

  uv_inform='(i4,1x,i2,1x,i2,1x,i2,1x,i2,1x,i2,1x,f5.2,1x,7x,f5.2,'
;            y     m     d     h     m     s    alat          mlt       
  uv_inform=uv_inform + '1x,i5,7x,f5.2,9x,f5.2)'
;                          n     ped    hall

; clear arrays

  ped=0
  hal=0
  lats=0
  mlts=0
  duts=0

; open file

  close,5
  openr,5,UVI_file

; read in number of lines in the file

  ntimes = 0L
  readf,5,format='(a7)',ntimes

; declare arrays

  d_uv_ut_arr=dblarr(ntimes)
  uv_alat_arr=fltarr(ntimes)
  uv_mlt_arr=fltarr(ntimes)
  uv_ped_arr=fltarr(ntimes)
  uv_hall_arr=fltarr(ntimes)

; initialize input variables

  y=0
  m=0
  d=0
  h=0
  min=0
  s=0

; read header line and ignore it

  spare=' '
  readf,5,spare		

; read in complete file

  tenpercent = ntimes/10

  for i=0l,ntimes-1 do begin

    if (indicate) and (i mod tenpercent eq 0) then 			$
      print, tostr(i*10/tenpercent), "% of the way through the UVI file"

    readf,5,format=uv_inform,y,m,d,h,min,s,alat,mlt,n,ped,hall
    c_a_to_r,[y,m,d,h,min,s],doubletime
    d_uv_ut_arr(i)=doubletime
    uv_alat_arr(i)=alat
    uv_mlt_arr(i)=mlt
    uv_ped_arr(i)=ped
    uv_hall_arr(i)=hall

  endfor

; if 1-D is requested, just shove arrays into returned arrays

  if (oned) then begin

    duts=d_uv_ut_arr
    lats=uv_alat_arr
    mlts=uv_mlt_arr
    ped=uv_ped_arr
    hal=uv_hall_arr

  endif else begin

; if 2-D is requested, then have to map points

; figure out how many times we have

    last_points = uniq(d_uv_ut_arr)
    nt = n_elements(last_points)

; add a -1 to make looping easier

    last_points = [-1,last_points]

; figure out Delta Latitude and minimum latitude

    dlat = abs(uv_alat_arr(1:499)-uv_alat_arr(0:498))
    loc = where(dlat gt 0.0, count)
    if count gt 0 then delta_lat = min(dlat(loc)) else delta_lat = 1.0
    min_lat = min(uv_alat_arr)
    dlat = delta_lat
    nlats = fix((90.0 - min_lat)/dlat) + 1

; figure out Delta MLT

    dmlt = abs(uv_mlt_arr(1:499)-uv_mlt_arr(0:498))
    loc = where(dmlt gt 0.0, count)
    if count gt 0 then delta_mlt = min(dmlt(loc)) else delta_mlt = 1.0
    dmlt = delta_mlt
    nmlts = fix(24.0/dmlt) + 1

; set latitudes and MLTs to return

    lats = 90.0 - findgen(nlats)*dlat
    mlts = findgen(nmlts)*dmlt

; declare return arrays

    duts = dblarr(nt)
    ped  = fltarr(nt,nmlts,nlats)
    hal  = fltarr(nt,nmlts,nlats)

; move 1-D parameters into 2-D arrays

    for n=0,nt-1 do begin

; current time

      duts(n) = d_uv_ut_arr(last_points(n)+1)

      for i=last_points(n)+1,last_points(n+1) do begin

; current lat and mlt

        l = fix((90.0 - uv_alat_arr(i))/dlat)
        m = fix((uv_mlt_arr(i))/dmlt)

; we want 24 MLT to go into the 0 MLT bin

        if m eq nmlts-1 then m = 0

; move values

	ped(n,m,l) = uv_ped_arr(i)
	hal(n,m,l) = uv_hall_arr(i)

      endfor

; now set 24 MLT = 0 MLT

      hal(n,nmlts-1,*) = hal(n,0,*)
      ped(n,nmlts-1,*) = ped(n,0,*)

    endfor

  endelse

  return

end


