;****************************************************************************************************
;+
; <PRO SAVE_BROWSER_EVENT>
; General description of routine
;
; @Param
;   event {in|out} {optional|required} {type=} {default=}
;     No Description
;
; @Author
;   Adam O'Connor
;
; @Copyright
;   Copyright &#169; 2006 ITT Industries. All rights reserved.
;
; @Hidden
;-
PRO SAVE_BROWSER_EVENT,event
COMPILE_OPT idl2,hidden
;error handling ... reset !ERROR_STATE:
MESSAGE,/RESET
;catch error:
CATCH,error
if (error NE 0) then begin ;error caught:
  ;obtain last message:
  HELP,/LAST_MESSAGE,OUTPUT=traceback
  messageStr=['Error Caught :','',traceback]
  ;display in dialog:
  void=DIALOG_MESSAGE(messageStr,/ERROR)
  ;terminate execution of this program:
  RETURN
endif
;obtain state structure for top-level-base from its uvalue:
WIDGET_CONTROL,event.top,GET_UVALUE=pState
;determine structure name for event:
eventName=STRUPCASE(TAG_NAMES(event,/STRUCTURE_NAME))
if (eventName EQ 'WIDGET_BASE') then begin ;widget base resize event:
  if (event.x EQ (*pState).tlbWidth) and (event.y EQ (*pState).tlbHeight) then RETURN
  ;reset widget size:
  WIDGET_CONTROL,event.top,XSIZE=(*pState).tlbWidth,YSIZE=(*pState).tlbHeight,$
                           XOFFSET=(*pState).xOffset,YOFFSET=0
  RETURN
endif
if (eventName EQ 'WIDGET_TREE_SEL') then begin ;widget tree event:
  ;determine what buttons are selected:
  spaceID=WIDGET_INFO(event.top,FIND_BY_UNAME='SPACE')
  spaceSet=WIDGET_INFO(spaceID,/BUTTON_SET)
  tabID=WIDGET_INFO(event.top,FIND_BY_UNAME='TAB')
  tabSet=WIDGET_INFO(tabID,/BUTTON_SET)
  commaID=WIDGET_INFO(event.top,FIND_BY_UNAME='COMMA')
  commaSet=WIDGET_INFO(commaID,/BUTTON_SET)
  binaryID=WIDGET_INFO(event.top,FIND_BY_UNAME='BINARY')
  binarySet=WIDGET_INFO(binaryID,/BUTTON_SET)
  treeID=WIDGET_INFO(event.top,FIND_BY_UNAME='TREE')
  totalIDs=WIDGET_INFO(treeID,/TREE_SELECT)
  saveID=WIDGET_INFO(event.top,FIND_BY_UNAME='SAVE')
  if ((*pState).restoreFlag EQ 1) then restoreID=WIDGET_INFO(event.top,FIND_BY_UNAME='RESTORE')
  ;set save button sensitivity accordingly:
  if (totalIDs[0] EQ -1) or (MAX([spaceSet,tabSet,commaSet,binarySet]) EQ 0) then begin
    WIDGET_CONTROL,saveID,SENSITIVE=0
    if ((*pState).restoreFlag EQ 1) then WIDGET_CONTROL,restoreID,SENSITIVE=0
    RETURN
  endif
  validIDs=BYTARR(N_ELEMENTS(totalIDs))
  for i=0,N_ELEMENTS(totalIDs)-1 do begin
    index=WHERE(*(*pState).dataTreeIDs EQ totalIDs[i],count)
    if (count NE 0) then validIDs[i]=1B
  endfor
  index=WHERE(validIDs EQ 1,count)
  if (count EQ 0) then begin
    WIDGET_CONTROL,saveID,SENSITIVE=0
    if ((*pState).restoreFlag EQ 1) then WIDGET_CONTROL,restoreID,SENSITIVE=0
  endif else begin
    if ((*pState).saveFlag EQ 1) then WIDGET_CONTROL,saveID,SENSITIVE=1
    if ((*pState).restoreFlag EQ 1) then WIDGET_CONTROL,restoreID,SENSITIVE=1
  endelse
  RETURN
endif
if (eventName EQ 'WIDGET_BUTTON') then begin ;widget checkbox event:
  ;determine what buttons are selected:
  spaceID=WIDGET_INFO(event.top,FIND_BY_UNAME='SPACE')
  spaceSet=WIDGET_INFO(spaceID,/BUTTON_SET)
  tabID=WIDGET_INFO(event.top,FIND_BY_UNAME='TAB')
  tabSet=WIDGET_INFO(tabID,/BUTTON_SET)
  commaID=WIDGET_INFO(event.top,FIND_BY_UNAME='COMMA')
  commaSet=WIDGET_INFO(commaID,/BUTTON_SET)
  binaryID=WIDGET_INFO(event.top,FIND_BY_UNAME='BINARY')
  binarySet=WIDGET_INFO(binaryID,/BUTTON_SET)
  buttonsSet=[binarySet,spaceSet,tabSet,commaSet]
  treeID=WIDGET_INFO(event.top,FIND_BY_UNAME='TREE')
  totalIDs=WIDGET_INFO(treeID,/TREE_SELECT)
  saveID=WIDGET_INFO(event.top,FIND_BY_UNAME='SAVE')
  if ((*pState).restoreFlag EQ 1) then restoreID=WIDGET_INFO(event.top,FIND_BY_UNAME='RESTORE')
  ;set save button sensitivity accordingly:
  if (totalIDs[0] EQ -1) or (MAX(buttonsSet) EQ 0) then begin
    WIDGET_CONTROL,saveID,SENSITIVE=0
    if ((*pState).restoreFlag EQ 1) then WIDGET_CONTROL,restoreID,SENSITIVE=0
    RETURN
  endif
  validIDs=BYTARR(N_ELEMENTS(totalIDs))
  for i=0,N_ELEMENTS(totalIDs)-1 do begin
    index=WHERE(*(*pState).dataTreeIDs EQ totalIDs[i],count)
    if (count NE 0) then validIDs[i]=1B
  endfor
  index=WHERE(validIDs EQ 1,count)
  if (count EQ 0) then begin
    WIDGET_CONTROL,saveID,SENSITIVE=0
    if ((*pState).restoreFlag EQ 1) then WIDGET_CONTROL,restoreID,SENSITIVE=0
  endif else begin
    if ((*pState).saveFlag EQ 1) then WIDGET_CONTROL,saveID,SENSITIVE=1
    if ((*pState).restoreFlag EQ 1) then WIDGET_CONTROL,restoreID,SENSITIVE=1
  endelse
endif
END
;****************************************************************************************************


;****************************************************************************************************
;+
; <PRO SAVE_BROWSER_RESTORE>
; General description of routine
;
; @Param
;   event {in|out} {optional|required} {type=} {default=}
;     No Description
;
; @Author
;   Adam O'Connor
;
; @Copyright
;   Copyright &#169; 2006 ITT Industries. All rights reserved.
;
; @Hidden
;-
PRO SAVE_BROWSER_RESTORE,event
COMPILE_OPT idl2,hidden
;error handling ... reset !ERROR_STATE:
MESSAGE,/RESET
;catch error:
CATCH,error
if (error NE 0) then begin ;error caught:
  case (!ERROR_STATE.NAME) of
    'IDL_M_CNTGETMEM' : $
      begin
        void=DIALOG_MESSAGE(['An error has occurred while attempting to restore and access one of the',$
                             'selected data variables.  There may not be enough free memory on the',$
                             'system in order to complete the desired operation.'])
        RETURN
      end
    'IDL_M_BADARRDIM' : $
      begin
        void=DIALOG_MESSAGE(['An error has occurred while attempting to restore and access one of the',$
                             'selected data variables.  There may not be enough free memory on the',$
                             'system in order to complete the desired operation.'])
        RETURN
      end
    'IDL_M_ARRAY_TOOMANYELTS' : $
      begin
        void=DIALOG_MESSAGE(['An error has occurred while attempting to restore and access one of the',$
                             'selected data variables.  There may not be enough free memory on the',$
                             'system in order to complete the desired operation.'])
        RETURN
      end
    else : $
      begin
        ;obtain last message:
        HELP,/LAST_MESSAGE,OUTPUT=traceback
        messageStr=['Error Caught :','',traceback]
        ;display in dialog:
        void=DIALOG_MESSAGE(messageStr,/ERROR)
        ;terminate execution of this program:
        RETURN
      end
  endcase
endif
;obtain state structure for top-level-base from its uvalue:
WIDGET_CONTROL,event.top,GET_UVALUE=pState
;determine what buttons are selected:
spaceID=WIDGET_INFO(event.top,FIND_BY_UNAME='SPACE')
spaceSet=WIDGET_INFO(spaceID,/BUTTON_SET)
tabID=WIDGET_INFO(event.top,FIND_BY_UNAME='TAB')
tabSet=WIDGET_INFO(tabID,/BUTTON_SET)
commaID=WIDGET_INFO(event.top,FIND_BY_UNAME='COMMA')
commaSet=WIDGET_INFO(commaID,/BUTTON_SET)
binaryID=WIDGET_INFO(event.top,FIND_BY_UNAME='BINARY')
binarySet=WIDGET_INFO(binaryID,/BUTTON_SET)
buttonsSet=[binarySet,spaceSet,tabSet,commaSet]
treeID=WIDGET_INFO(event.top,FIND_BY_UNAME='TREE')
totalIDs=WIDGET_INFO(treeID,/TREE_SELECT)
validIDs=BYTARR(N_ELEMENTS(totalIDs))
for i=0,N_ELEMENTS(totalIDs)-1 do begin
  index=WHERE(*(*pState).dataTreeIDs EQ totalIDs[i],count)
  if (count NE 0) then validIDs[i]=1B
endfor
index=WHERE(validIDs EQ 1,count)
if (count EQ 0) then RETURN
WIDGET_CONTROL,/HOURGLASS
;determine what nodes within tree widget are selected and sort:
selectedIDs=totalIDs[index]
selectedIDs=selectedIDs[SORT(selectedIDs)]
selectedIDs=selectedIDs[UNIQ(selectedIDs)]
;get a listing of the variables that currently exist at the $MAIN$ level:
mainVars=STRUPCASE(SCOPE_VARNAME(LEVEL=1))
badStructureNames=['VOID','SAVEINDEX','STRUCTNAME','QUERY','MAINVARS',$
                   'VARNAME','VARSTART','PSTATE','SELECTEDIDS','BADSTRUCTURENAMES']
FOR i=0,N_ELEMENTS(selectedIDs)-1 do begin
  ;extract variable name:
  WIDGET_CONTROL,selectedIDs[i],GET_UVALUE=varName
  ;determine where the variable name starts (variable names cannot start with numbers):
  varStart=0
  while ((BYTE(STRMID(varName,varStart,1)))[0] GE 48) and ((BYTE(STRMID(varName,varStart,1)))[0] LE 57) do begin
    varStart=varStart+1
  endwhile
  ;extract save file index:
  saveIndex=LONG(STRMID(varName,0,varStart))
  ;check to see if item is structure tag:
  if (STRPOS(varName,'.') NE -1) then begin ;structure tag:
    ;parse-out the parent structure name:
    structName=STRMID(varName,varStart,STRPOS(varName,'.')-1)
    ;make sure that name of structure about to be restored does not conflict with any of the local
    ;variables that are used within this program module:
    void=WHERE(badStructureNames EQ (STRUPCASE(structName)),badStructureCount)
    if (badStructureCount NE 0) then begin
      void=DIALOG_MESSAGE(['The name of the selected structure conflicts with one of the variables used',$
                           'internally by this program :','','"'+structName+'"','',$
                           'In order to access the data contained in this structure you must either output',$
                           'the entire structure or restore the selected save file into the IDL software',$
                           'package and obtain the data using the appropriate methods.'])
      CONTINUE
    endif
    ;parse out the tag name:
    save_browser_tagName=STRMID(varName,STRPOS(varName,'.')+1)
    if (STRPOS(save_browser_tagName,'.') NE -1) then begin ;nested:
      ;parse out first and second levels of sub-tags:
      save_browser_subName=STRMID(save_browser_tagName,0,STRPOS(save_browser_tagName,'.'))
      save_browser_subTagName=STRMID(save_browser_tagName,STRPOS(save_browser_tagName,'.')+1)
      ;check to see if variable already exists:
      void=WHERE(mainVars EQ (STRUPCASE(save_browser_subTagName)),mainVarCount)
      if (mainVarCount NE 0) then begin
        query=DIALOG_MESSAGE(['The following data variable already exists at the $MAIN$ IDL level :',$
                              '','"'+save_browser_subTagName+'"','',$
                              'Do you wish to overwrite the value for this variable ?'],/QUESTION)
        if (STRUPCASE(query) EQ 'NO') then CONTINUE
      endif
      ;restore the entire structure:
      (*(*pState).oSaveFiles)[saveIndex]->RESTORE,structName
      ;obtain the structure into a new variable that can be accessed:
      save_browser_structure=(SCOPE_VARFETCH(structName,LEVEL=0))
      ;determine that appropriate tag indices:
      save_browser_tags=STRUPCASE(TAG_NAMES(save_browser_structure))
      save_browser_tagIndex=(WHERE(save_browser_tags EQ save_browser_subName))[0]
      save_browser_subTags=STRUPCASE(TAG_NAMES(save_browser_structure.(save_browser_tagIndex)))
      save_browser_subIndex=(WHERE(save_browser_subTags EQ save_browser_subTagName))[0]
      ;extract the relevant data:
      data=save_browser_structure.(save_browser_tagIndex).(save_browser_subIndex)
      (SCOPE_VARFETCH(save_browser_subTagName,/ENTER,LEVEL=1)) = TEMPORARY(data)
      ;loop through the rest of the tags in the structure that was just restored, and cleanup any heap memory:
      save_browser_nTags=N_TAGS(save_browser_structure)
      for j=0,save_browser_nTags-1 do begin
        if (j EQ save_browser_tagIndex) then CONTINUE
        HEAP_FREE,save_browser_structure.(j)
      endfor
      save_browser_nTags=N_TAGS(save_browser_structure.(save_browser_tagIndex))
      for j=0,save_browser_nTags-1 do begin
        if (j EQ save_browser_subIndex) then CONTINUE
        HEAP_FREE,save_browser_structure.(save_browser_tagIndex).(j)
      endfor
    endif else begin ;single sub-tag:
      ;check to see if variable already exists:
      void=WHERE(mainVars EQ (STRUPCASE(save_browser_tagName)),mainVarCount)
      if (mainVarCount NE 0) then begin
        query=DIALOG_MESSAGE(['The following data variable already exists at the $MAIN$ IDL level :',$
                              '','"'+save_browser_tagName+'"','',$
                              'Do you wish to overwrite the value for this variable ?'],/QUESTION)
        if (STRUPCASE(query) EQ 'NO') then CONTINUE
      endif
      ;restore the entire structure:
      (*(*pState).oSaveFiles)[saveIndex]->RESTORE,structName
      ;obtain the structure into a new variable that can be accessed:
      save_browser_structure=(SCOPE_VARFETCH(structName,LEVEL=0))
      ;determine that appropriate tag indices:
      save_browser_tags=STRUPCASE(TAG_NAMES(save_browser_structure))
      save_browser_index=(WHERE(save_browser_tags EQ save_browser_tagName))[0]
      ;extract the relevant data:
      data=save_browser_structure.(save_browser_index)
      (SCOPE_VARFETCH(save_browser_tagName,/ENTER,LEVEL=1)) = TEMPORARY(data)
      ;loop through the rest of the tags in the structure that was just restored, and cleanup any heap memory:
      save_browser_nTags=N_TAGS(save_browser_structure)
      for j=0,save_browser_nTags-1 do begin
        if (j EQ save_browser_index) then CONTINUE
        HEAP_FREE,save_browser_structure.(j)
      endfor
    endelse
  endif else begin ;normal variable:
    varName=STRMID(varName,varStart)
    ;check to see if variable already exists:
    void=WHERE(mainVars EQ (STRUPCASE(varName)),mainVarCount)
    if (mainVarCount NE 0) then begin
      query=DIALOG_MESSAGE(['The following data variable already exists at the $MAIN$ IDL level :',$
                            '','"'+varName+'"','',$
                            'Do you wish to overwrite the value for this variable ?'],/QUESTION)
      if (STRUPCASE(query) EQ 'NO') then CONTINUE
    endif
    ;restore the selected variable:
    (*(*pState).oSaveFiles)[saveIndex]->RESTORE,varName
    ;insert data into a new variable at the $MAIN$ level:
    (SCOPE_VARFETCH(varName,/ENTER,LEVEL=1)) = (SCOPE_VARFETCH(varName,LEVEL=0))
  endelse
ENDFOR
restoreID=WIDGET_INFO(event.top,FIND_BY_UNAME='RESTORE')
WIDGET_CONTROL,restoreID,SENSITIVE=0
END
;****************************************************************************************************


;****************************************************************************************************
;+
; <PRO SAVE_BROWSER_SAVE>
; General description of routine
;
; @Param
;   event {in|out} {optional|required} {type=} {default=}
;     No Description
;
; @Author
;   Adam O'Connor
;
; @Copyright
;   Copyright &#169; 2006 ITT Industries. All rights reserved.
;
; @Hidden
;-
PRO SAVE_BROWSER_SAVE,event
COMPILE_OPT idl2,hidden
;error handling ... reset !ERROR_STATE:
MESSAGE,/RESET
;catch error:
CATCH,error
if (error NE 0) then begin ;error caught:
  case (!ERROR_STATE.NAME) of
    'IDL_M_NOCON_PTR' : $
      begin
        void=DIALOG_MESSAGE(['The following structure variable contains pointers and/or objects :',$
                             '','"'+varName+'"','',$
                             'The output of entire structures that contain pointers and/or object',$
                             'references to the Structured Binary (*.bin) output file format is not',$
                             'supported by this program.  In order to output the heap variable',$
                             'data referenced by any pointers you must expand the structure tree',$
                             'and select the individual structure tags for the desired variables.'])
        ;if current output file is still open, close it:
        if (SIZE(lun,/TYPE) NE 0) then FREE_LUN,TEMPORARY(lun)
        ;if status report dialog is still active, destroy it:
        if (SIZE(tlb,/TYPE) NE 0) then WIDGET_CONTROL,TEMPORARY(tlb),/DESTROY
        if (SIZE(dataInfo,/TYPE) NE 0) and (SIZE(varName,/TYPE) NE 0) and (SIZE(extension,/TYPE) NE 0) then begin
          if (dataInfo.n_elements GT 1) and (SIZE(s,/TYPE) NE 0) then $
            FILE_DELETE,varName+STRTRIM(s+1,2)+extension,/QUIET $
          else $
            FILE_DELETE,varName+extension,/QUIET
        endif
        if (SIZE(saveID,/TYPE) NE 0) then WIDGET_CONTROL,saveID,SENSITIVE=0
        ;cleanup heap memory just in case:
        if (SIZE(save_browser_data,/TYPE) NE 0) then HEAP_FREE,TEMPORARY(save_browser_data)
        if (SIZE(save_browser_structure,/TYPE) NE 0) and (SIZE(structRestoreFlag,/TYPE) NE 0) then begin
          if (structRestoreFlag EQ 1) then begin
            HEAP_FREE,TEMPORARY(save_browser_structure)
            HEAP_FREE,TEMPORARY((SCOPE_VARFETCH(structName,LEVEL=0)))
          endif
        endif
        ;re-create IDL_SAVEFILE object just in case:
        OBJ_DESTROY,(*(*pState).oSaveFiles)[saveIndex]
        (*(*pState).oSaveFiles)[saveIndex]=OBJ_NEW('IDL_SAVEFILE',(*pState).files[saveIndex],$
                                                   /RELAXED_STRUCTURE_ASSIGNMENT)
        RETURN
      end
    'IDL_M_CNTGETMEM' : $
      begin
        void=DIALOG_MESSAGE(['An error has occurred while attempting to restore and access one of the',$
                             'selected data variables.  There may not be enough free memory on the',$
                             'system in order to complete the desired operation.'])
        ;if current output file is still open, close it:
        if (SIZE(lun,/TYPE) NE 0) then FREE_LUN,TEMPORARY(lun)
        ;if status report dialog is still active, destroy it:
        if (SIZE(tlb,/TYPE) NE 0) then WIDGET_CONTROL,TEMPORARY(tlb),/DESTROY
        if (SIZE(varName,/TYPE) NE 0) and (SIZE(extension,/TYPE) NE 0) then begin
          if (SIZE(s,/TYPE) NE 0) then FILE_DELETE,varName+STRTRIM(s+1,2)+extension,/QUIET
          FILE_DELETE,varName+extension,/QUIET
        endif
        if (SIZE(saveID,/TYPE) NE 0) then WIDGET_CONTROL,saveID,SENSITIVE=0
        ;cleanup heap memory just in case:
        if (SIZE(save_browser_data,/TYPE) NE 0) then HEAP_FREE,TEMPORARY(save_browser_data)
        if (SIZE(save_browser_structure,/TYPE) NE 0) and (SIZE(structRestoreFlag,/TYPE) NE 0) then begin
          if (structRestoreFlag EQ 1) then begin
            HEAP_FREE,TEMPORARY(save_browser_structure)
            HEAP_FREE,TEMPORARY((SCOPE_VARFETCH(structName,LEVEL=0)))
          endif
        endif
        ;re-create IDL_SAVEFILE object just in case:
        OBJ_DESTROY,(*(*pState).oSaveFiles)[saveIndex]
        (*(*pState).oSaveFiles)[saveIndex]=OBJ_NEW('IDL_SAVEFILE',(*pState).files[saveIndex],$
                                                   /RELAXED_STRUCTURE_ASSIGNMENT)
        RETURN
      end
    'IDL_M_BADARRDIM' : $
      begin
        void=DIALOG_MESSAGE(['An error has occurred while attempting to restore and access one of the',$
                             'selected data variables.  There may not be enough free memory on the',$
                             'system in order to complete the desired operation.'])
        ;if current output file is still open, close it:
        if (SIZE(lun,/TYPE) NE 0) then FREE_LUN,TEMPORARY(lun)
        ;if status report dialog is still active, destroy it:
        if (SIZE(tlb,/TYPE) NE 0) then WIDGET_CONTROL,TEMPORARY(tlb),/DESTROY
        if (SIZE(varName,/TYPE) NE 0) and (SIZE(extension,/TYPE) NE 0) then begin
          if (SIZE(s,/TYPE) NE 0) then FILE_DELETE,varName+STRTRIM(s+1,2)+extension,/QUIET
          FILE_DELETE,varName+extension,/QUIET
        endif
        if (SIZE(saveID,/TYPE) NE 0) then WIDGET_CONTROL,saveID,SENSITIVE=0
        ;cleanup heap memory just in case:
        if (SIZE(save_browser_data,/TYPE) NE 0) then HEAP_FREE,TEMPORARY(save_browser_data)
        if (SIZE(save_browser_structure,/TYPE) NE 0) and (SIZE(structRestoreFlag,/TYPE) NE 0) then begin
          if (structRestoreFlag EQ 1) then begin
            HEAP_FREE,TEMPORARY(save_browser_structure)
            HEAP_FREE,TEMPORARY((SCOPE_VARFETCH(structName,LEVEL=0)))
          endif
        endif
        ;re-create IDL_SAVEFILE object just in case:
        OBJ_DESTROY,(*(*pState).oSaveFiles)[saveIndex]
        (*(*pState).oSaveFiles)[saveIndex]=OBJ_NEW('IDL_SAVEFILE',(*pState).files[saveIndex],$
                                                   /RELAXED_STRUCTURE_ASSIGNMENT)
        RETURN
      end
    'IDL_M_ARRAY_TOOMANYELTS' : $
      begin
        void=DIALOG_MESSAGE(['An error has occurred while attempting to restore and access one of the',$
                             'selected data variables.  There may not be enough free memory on the',$
                             'system in order to complete the desired operation.'])
        ;if current output file is still open, close it:
        if (SIZE(lun,/TYPE) NE 0) then FREE_LUN,TEMPORARY(lun)
        ;if status report dialog is still active, destroy it:
        if (SIZE(tlb,/TYPE) NE 0) then WIDGET_CONTROL,TEMPORARY(tlb),/DESTROY
        if (SIZE(varName,/TYPE) NE 0) and (SIZE(extension,/TYPE) NE 0) then begin
          if (SIZE(s,/TYPE) NE 0) then FILE_DELETE,varName+STRTRIM(s+1,2)+extension,/QUIET
          FILE_DELETE,varName+extension,/QUIET
        endif
        if (SIZE(saveID,/TYPE) NE 0) then WIDGET_CONTROL,saveID,SENSITIVE=0
        ;cleanup heap memory just in case:
        if (SIZE(save_browser_data,/TYPE) NE 0) then HEAP_FREE,TEMPORARY(save_browser_data)
        if (SIZE(save_browser_structure,/TYPE) NE 0) and (SIZE(structRestoreFlag,/TYPE) NE 0) then begin
          if (structRestoreFlag EQ 1) then begin
            HEAP_FREE,TEMPORARY(save_browser_structure)
            HEAP_FREE,TEMPORARY((SCOPE_VARFETCH(structName,LEVEL=0)))
          endif
        endif
        ;re-create IDL_SAVEFILE object just in case:
        OBJ_DESTROY,(*(*pState).oSaveFiles)[saveIndex]
        (*(*pState).oSaveFiles)[saveIndex]=OBJ_NEW('IDL_SAVEFILE',(*pState).files[saveIndex],$
                                                   /RELAXED_STRUCTURE_ASSIGNMENT)
        RETURN
      end
    else : $
      begin
        ;obtain last message:
        HELP,/LAST_MESSAGE,OUTPUT=traceback
        messageStr=['Error Caught :','',traceback]
        ;display in dialog:
        void=DIALOG_MESSAGE(messageStr,/ERROR)
        ;if current output file is still open, close it:
        if (SIZE(lun,/TYPE) NE 0) then FREE_LUN,TEMPORARY(lun)
        ;if status report dialog is still active, destroy it:
        if (SIZE(tlb,/TYPE) NE 0) then WIDGET_CONTROL,TEMPORARY(tlb),/DESTROY
        if (SIZE(saveID,/TYPE) NE 0) then WIDGET_CONTROL,saveID,SENSITIVE=0
        ;cleanup heap memory just in case:
        if (SIZE(save_browser_data,/TYPE) NE 0) then HEAP_FREE,TEMPORARY(save_browser_data)
        if (SIZE(save_browser_structure,/TYPE) NE 0) and (SIZE(structRestoreFlag,/TYPE) NE 0) then begin
          if (structRestoreFlag EQ 1) then begin
            HEAP_FREE,TEMPORARY(save_browser_structure)
            HEAP_FREE,TEMPORARY((SCOPE_VARFETCH(structName,LEVEL=0)))
          endif
        endif
        ;re-create IDL_SAVEFILE object just in case:
        OBJ_DESTROY,(*(*pState).oSaveFiles)[saveIndex]
        (*(*pState).oSaveFiles)[saveIndex]=OBJ_NEW('IDL_SAVEFILE',(*pState).files[saveIndex],$
                                                   /RELAXED_STRUCTURE_ASSIGNMENT)
        ;terminate execution of this program:
        RETURN
      end
  endcase
endif
;obtain state structure for top-level-base from its uvalue:
WIDGET_CONTROL,event.top,GET_UVALUE=pState
;determine what buttons are selected:
saveID=WIDGET_INFO(event.top,FIND_BY_UNAME='SAVE')
spaceID=WIDGET_INFO(event.top,FIND_BY_UNAME='SPACE')
spaceSet=WIDGET_INFO(spaceID,/BUTTON_SET)
tabID=WIDGET_INFO(event.top,FIND_BY_UNAME='TAB')
tabSet=WIDGET_INFO(tabID,/BUTTON_SET)
commaID=WIDGET_INFO(event.top,FIND_BY_UNAME='COMMA')
commaSet=WIDGET_INFO(commaID,/BUTTON_SET)
binaryID=WIDGET_INFO(event.top,FIND_BY_UNAME='BINARY')
binarySet=WIDGET_INFO(binaryID,/BUTTON_SET)
buttonsSet=[binarySet,spaceSet,tabSet,commaSet]
treeID=WIDGET_INFO(event.top,FIND_BY_UNAME='TREE')
totalIDs=WIDGET_INFO(treeID,/TREE_SELECT)
validIDs=BYTARR(N_ELEMENTS(totalIDs))
for i=0,N_ELEMENTS(totalIDs)-1 do begin
  index=WHERE(*(*pState).dataTreeIDs EQ totalIDs[i],count)
  if (count NE 0) then validIDs[i]=1B
endfor
index=WHERE(validIDs EQ 1,count)
if (count EQ 0) then RETURN
;display a status report dialog:
tlb=WIDGET_BASE(TITLE='Status Report',/COLUMN,/ALIGN_CENTER,TLB_FRAME_ATTR=19)
  spacer=WIDGET_LABEL(tlb,VALUE=' ')
  label=WIDGET_LABEL(tlb,VALUE='     EXPORTING SELECTED DATA VARIABLES     ')
  spacer=WIDGET_LABEL(tlb,VALUE=' ')
  label=WIDGET_LABEL(tlb,VALUE='****** PLEASE WAIT ******')
  spacer=WIDGET_LABEL(tlb,VALUE=' ')
geom=WIDGET_INFO(tlb,/GEOMETRY)
xHalfSize=geom.Scr_XSize/2
yHalfSize=geom.Scr_YSize/2
WIDGET_CONTROL,tlb,XOFFSET=(*pState).xCenter-xHalfSize,YOFFSET=(*pState).yCenter-yHalfSize
WIDGET_CONTROL,tlb,/REALIZE
WIDGET_CONTROL,/HOURGLASS
;determine what nodes within tree widget are selected and sort:
selectedIDs=totalIDs[index]
selectedIDs=selectedIDs[SORT(selectedIDs)]
selectedIDs=selectedIDs[UNIQ(selectedIDs)]
dimWarnFlag=0B
objWarnFlag=0B
structWarnFlag=0B
structPtrFlag=0B
structObjFlag=0B
structBinFlag=0B
badStructureNames=['VOID','DATAINFO','STRUCTRESTOREFLAG','SAVEINDEX','OBJWARNFLAG','STRUCTNAME',$
                   'VARNAME','VARSTART','PSTATE','SELECTEDIDS','SAVEID','TLB','BUTTONSSET',$
                   'DIMWARNFLAG','OBJWARNFLAG','STRUCTWARNFLAG','STRUCTPTRFLAG','STRUCTOBJFLAG',$
                   'STRUCTBINFLAG','BADSTRUCTURENAMES']
FOR i=0,N_ELEMENTS(selectedIDs)-1 do begin
  ;extract variable name:
  WIDGET_CONTROL,selectedIDs[i],GET_UVALUE=varName
  ;determine where the variable name starts (variable names cannot start with numbers):
  varStart=0
  while ((BYTE(STRMID(varName,varStart,1)))[0] GE 48) and ((BYTE(STRMID(varName,varStart,1)))[0] LE 57) do begin
    varStart=varStart+1
  endwhile
  ;extract save file index:
  saveIndex=LONG(STRMID(varName,0,varStart))
  structRestoreFlag=0B
  ;check to see if item is structure tag:
  if (STRPOS(varName,'.') NE -1) then begin ;structure tag:
    ;parse-out the parent structure name:
    structName=STRMID(varName,varStart,STRPOS(varName,'.')-1)
    ;make sure that name of structure about to be restored does not conflict with any of the local
    ;variables that are used within this program module until the structure variable is destroyed:
    void=WHERE(badStructureNames EQ (STRUPCASE(structName)),badStructureCount)
    if (badStructureCount NE 0) then begin
      void=DIALOG_MESSAGE(['The name of the selected structure conflicts with one of the variables used',$
                           'internally by this program :','','"'+structName+'"','',$
                           'In order to access the data contained in this structure you must either output',$
                           'the entire structure or restore the selected save file into the IDL software',$
                           'package and obtain the data using the appropriate methods.'])
      CONTINUE
    endif
    ;restore the entire structure:
    (*(*pState).oSaveFiles)[saveIndex]->RESTORE,structName
    structRestoreFlag=1B
    ;obtain the structure into a new variable that can be accessed:
    save_browser_structure=(SCOPE_VARFETCH(structName,LEVEL=0))
    ;parse out the tag name:
    save_browser_tagName=STRMID(varName,STRPOS(varName,'.')+1)
    if (STRPOS(save_browser_tagName,'.') NE -1) then begin ;nested:
      ;parse out first and second levels of sub-tags:
      save_browser_subName=STRMID(save_browser_tagName,0,STRPOS(save_browser_tagName,'.'))
      save_browser_subTagName=STRMID(save_browser_tagName,STRPOS(save_browser_tagName,'.')+1)
      save_browser_tags=STRUPCASE(TAG_NAMES(save_browser_structure))
      save_browser_tagIndex=(WHERE(save_browser_tags EQ save_browser_subName))[0]
      save_browser_subTags=STRUPCASE(TAG_NAMES(save_browser_structure.(save_browser_tagIndex)))
      save_browser_subIndex=(WHERE(save_browser_subTags EQ save_browser_subTagName))[0]
      ;extract the relevant data:
      save_browser_data=save_browser_structure.(save_browser_tagIndex).(save_browser_subIndex)
    endif else begin ;single sub-tag:
      save_browser_tags=STRUPCASE(TAG_NAMES(save_browser_structure))
      save_browser_index=(WHERE(save_browser_tags EQ save_browser_tagName))[0]
      ;extract the relevant data:
      save_browser_data=save_browser_structure.(save_browser_index)
    endelse
    varName=STRMID(varName,varStart)
  endif else begin ;normal variable:
    varName=STRMID(varName,varStart)
    ;restore the selected variable:
    (*(*pState).oSaveFiles)[saveIndex]->RESTORE,varName
    ;obtain the data into a new variable that can be accessed:
    save_browser_data=TEMPORARY((SCOPE_VARFETCH(varName,LEVEL=0)))
  endelse
  ;gather useful information about data being output:
  dataInfo=SIZE(save_browser_data,/STRUCTURE)
  ;display warning message if variable is an object reference and continue:
  if (dataInfo.Type EQ 11) then begin
    if (objWarnFlag EQ 0) then begin
      void=DIALOG_MESSAGE(['The output of instance data associated with an object is not supported',$
                           'by this program.  In order to access the data associated with the :',$
                           '','"'+varName+'"','',$
                           'object you must restore the selected save file into the IDL software',$
                           'package and obtain the data using the appropriate methods.'])
      objWarnFlag=1B
    endif
    ;cleanup heap memory:
    HEAP_FREE,save_browser_data
    if (structRestoreFlag EQ 1) then begin
      HEAP_FREE,TEMPORARY(save_browser_structure)
      HEAP_FREE,TEMPORARY((SCOPE_VARFETCH(structName,LEVEL=0)))
    endif
    ;re-create IDL_SAVEFILE object so heap variable data can be accessed in the future:
    OBJ_DESTROY,(*(*pState).oSaveFiles)[saveIndex]
    (*(*pState).oSaveFiles)[saveIndex]=OBJ_NEW('IDL_SAVEFILE',(*pState).files[saveIndex],$
                                               /RELAXED_STRUCTURE_ASSIGNMENT)
    CONTINUE
  endif
  ;if data is a valid pointer reference then dereference in order to obtain heap variable:
  if (dataInfo.Type EQ 10) then begin
    if (dataInfo.n_dimensions GT 0) then begin ;pointer array:
      ;reform array of pointers into a structure:
      if (PTR_VALID(save_browser_data[0])) then begin
        if (SIZE(*save_browser_data[0],/TYPE) NE 0) then $
          save_browser_ptrStruct=CREATE_STRUCT('POINTER1',*save_browser_data[0]) $
        else $
          save_browser_ptrStruct=CREATE_STRUCT('POINTER1','<Undefined>')
      endif else begin
        save_browser_ptrStruct=CREATE_STRUCT('POINTER1','<NullPointer>')
      endelse
      if (dataInfo.n_elements GT 1) then begin
        for save_browser_j=1,dataInfo.n_elements-1 do begin
          if (PTR_VALID(save_browser_data[save_browser_j])) then begin
            if (SIZE(*save_browser_data[save_browser_j],/TYPE) NE 0) then $
              save_browser_ptrStruct=CREATE_STRUCT(save_browser_ptrStruct,'POINTER'+STRTRIM(save_browser_j+1,2),*save_browser_data[save_browser_j]) $
            else $
              save_browser_ptrStruct=CREATE_STRUCT(save_browser_ptrStruct,'POINTER'+STRTRIM(save_browser_j+1,2),'<Undefined>')
          endif else begin
            save_browser_ptrStruct=CREATE_STRUCT(save_browser_ptrStruct,'POINTER'+STRTRIM(save_browser_j+1,2),'<NullPointer>')
          endelse
        endfor
      endif
      ;cleanup heap memory:
      HEAP_FREE,TEMPORARY(save_browser_data)
      ;re-create IDL_SAVEFILE object so heap variable data can be accessed in the future:
      OBJ_DESTROY,(*(*pState).oSaveFiles)[saveIndex]
      (*(*pState).oSaveFiles)[saveIndex]=OBJ_NEW('IDL_SAVEFILE',(*pState).files[saveIndex],$
                                                 /RELAXED_STRUCTURE_ASSIGNMENT)
      save_browser_data=TEMPORARY(save_browser_ptrStruct)
      dataInfo=SIZE(save_browser_data,/STRUCTURE)
      ;make sure structure does not contain any pointers:
      save_browser_ptrFlag=0B
      for save_browser_j=0,N_TAGS(save_browser_data)-1 do begin
        if (SIZE(save_browser_data.(save_browser_j),/TYPE) EQ 10) then save_browser_ptrFlag=1B
      endfor
      if (save_browser_ptrFlag EQ 1) then begin
        void=DIALOG_MESSAGE(['The following variable contains pointers to pointers :',$
                             '','"'+varName+'"','',$
                             'The output of data stored in nested pointers is not supported by this',$
                             'program.  In order to access this data you must restore the selected',$
                             'save file into the IDL software package and obtain the data using the',$
                             'appropriate methods.'])
        if (structRestoreFlag EQ 1) then begin
          HEAP_FREE,TEMPORARY(save_browser_structure)
          HEAP_FREE,TEMPORARY((SCOPE_VARFETCH(structName,LEVEL=0)))
        endif
        CONTINUE
      endif
    endif else begin ;scalar pointer:
      if (PTR_VALID(save_browser_data)) then begin
        if (SIZE(*save_browser_data,/TYPE) NE 0) then begin
          save_browser_temp = *save_browser_data
          ;cleanup heap memory:
          HEAP_FREE,TEMPORARY(save_browser_data)
          save_browser_data=TEMPORARY(save_browser_temp)
        endif else begin
          ;cleanup heap memory:
          HEAP_FREE,TEMPORARY(save_browser_data)
          save_browser_data='<Undefined>'
        endelse
      endif else begin
        save_browser_data='<NullPointer>'
      endelse
      ;re-create IDL_SAVEFILE object so heap variable data can be accessed in the future:
      OBJ_DESTROY,(*(*pState).oSaveFiles)[saveIndex]
      (*(*pState).oSaveFiles)[saveIndex]=OBJ_NEW('IDL_SAVEFILE',(*pState).files[saveIndex],$
                                                 /RELAXED_STRUCTURE_ASSIGNMENT)
      dataInfo=SIZE(save_browser_data,/STRUCTURE)
      ;check for nested pointers:
      if (dataInfo.Type EQ 10) then begin
        void=DIALOG_MESSAGE(['The following variable contains pointers to pointers :',$
                             '','"'+varName+'"','',$
                             'The output of data stored in nested pointers is not supported by this',$
                             'program.  In order to access this data you must restore the selected',$
                             'save file into the IDL software package and obtain the data using',$
                             'the appropriate methods.'])
        if (structRestoreFlag EQ 1) then begin
          HEAP_FREE,TEMPORARY(save_browser_structure)
          HEAP_FREE,TEMPORARY((SCOPE_VARFETCH(structName,LEVEL=0)))
        endif
        CONTINUE
      endif
    endelse
  endif
  if (structRestoreFlag EQ 1) then begin
    HEAP_FREE,TEMPORARY(save_browser_structure)
    HEAP_FREE,TEMPORARY((SCOPE_VARFETCH(structName,LEVEL=0)))
  endif
  ;loop through each of the 4 output formats:
  for j=0,3 do begin
    ;if checkbox is not selected for current format, immediately skip to next:
    if (buttonsSet[j] EQ 0) then CONTINUE
    if (dataInfo.type NE 8) and (dataInfo.n_dimensions GT 3) and (j GE 2) then begin
      if (dimWarnFlag EQ 0) then begin
        void=DIALOG_MESSAGE(['The output of arrays with 4 or more dimensions to Tab Delimited (*.tab) or',$
                             'Comma Separated (*.csv) ASCII text format files is not supported by this',$
                             'program.  The supported output formats for the data stored in the variable :',$
                             '','"'+varName+'"','',$
                             'are the Space Delimited ASCII (*.txt) and Structured Binary (*.bin) formats.'])
        dimWarnFlag=1B
      endif
      CONTINUE
    endif
    if (dataInfo.type EQ 8) and (j GE 2) then begin
      if (structWarnFlag EQ 0) then begin
        void=DIALOG_MESSAGE(['The output of entire structures or pointer arrays to Tab Delimited (*.tab)',$
                             'or Comma Separated (*.csv) ASCII text format files is not supported by this',$
                             'program.  The supported output formats for the data stored in the variable :',$
                             '','"'+varName+'"','',$
                             'are the Space Delimited ASCII (*.txt) and Structured Binary (*.bin) formats.'])
        structWarnFlag=1B
      endif
      ;cleanup heap memory if necessary:
      if (buttonsSet[0] EQ 0) and (buttonsSet[1] EQ 0) then begin
        HEAP_FREE,save_browser_data
        ;re-create IDL_SAVEFILE object so heap variable data can be accessed in the future:
        OBJ_DESTROY,(*(*pState).oSaveFiles)[saveIndex]
        (*(*pState).oSaveFiles)[saveIndex]=OBJ_NEW('IDL_SAVEFILE',(*pState).files[saveIndex],$
                                                   /RELAXED_STRUCTURE_ASSIGNMENT)
      endif
      CONTINUE
    endif
    binaryFlag=0B
    case j of
      0 : begin
            extension='.bin'
            binaryFlag=1B
          end
      1 : begin
            extension='.txt'
            separator=' '
          end
      2 : begin
            extension='.tab'
            separator='	'
          end
      3 : begin
            extension='.csv'
            separator=','
          end
    endcase
    ;check to see if the default output file already exists?:
    outFile = varName + extension
    if (FILE_TEST(outFile)) then begin
      defExt=STRMID(extension,1)
      title='Default output file "'+outFile+'" already exists : Specify different filename'
      newFile=DIALOG_PICKFILE(TITLE=title,DEFAULT_EXTENSION=defExt,DIALOG_PARENT=event.top,/WRITE,$
                              /OVERWRITE_PROMPT,FILTER='*'+extension)
      if (newFile EQ '') then CONTINUE ;user hit the "Cancel" button:
      outFile = newFile
    endif
    ;open the default output filename for writing:
    OPENW,lun,outFile,/GET
    if (dataInfo.type NE 8) then begin ;everything other than a structure:
      case dataInfo.n_dimensions of
        0 : $
          begin ;scalar:
            if (binaryFlag EQ 0) then begin ;ASCII output:
              ;convert byte data to integer so it doesn't get output as ASCII characters instead of numbers:
              if (dataInfo.Type EQ 1) then save_browser_data=FIX(TEMPORARY(save_browser_data))
              ;deal with double-precision, complex, and dcomplex data types:
              if (dataInfo.Type EQ 5) or (dataInfo.Type EQ 6) or (dataInfo.Type EQ 9) then begin
                if (dataInfo.Type EQ 5) then begin ;double:
                  maxData=STRTRIM(MAX(save_browser_data,MIN=minData),2)
                  maxData=STRMID(maxData,0,STRPOS(maxData,'.')+1)
                  decimals=14-STRLEN(maxData)
                  if (minData LT 0) then decimals=decimals-1
                  PRINTF,lun,STRTRIM(STRING(save_browser_data,FORMAT='(D14.'+STRTRIM(decimals,2)+')'),2)
                endif
                if (dataInfo.Type EQ 6) then begin ;complex:
                  PRINTF,lun,'('+STRTRIM(FLOAT(save_browser_data),2)+', '+STRTRIM(IMAGINARY(save_browser_data),2)+')'
                endif
                if (dataInfo.Type EQ 9) then begin ;dcomplex:
                  maxReal=MAX(DOUBLE(save_browser_data),MIN=minReal)
                  maxImag=MAX(IMAGINARY(save_browser_data),MIN=minImag)
                  maxData=maxReal>maxImag
                  minData=minReal<minImag
                  maxData=STRTRIM(maxData,2)
                  maxData=STRMID(maxData,0,STRPOS(maxData,'.')+1)
                  decimals=14-STRLEN(maxData)
                  if (minData LT 0) then decimals=decimals-1
                  PRINTF,lun,'('+STRTRIM(STRING(DOUBLE(save_browser_data),FORMAT='(D14.'+STRTRIM(decimals,2)+')'),2)+$
                    ', '+STRTRIM(STRING(IMAGINARY(save_browser_data),FORMAT='(D14.'+STRTRIM(decimals,2)+')'),2)+')'
                endif
              endif else begin ;everything else:
                PRINTF,lun,STRTRIM(save_browser_data,2)
              endelse
            endif else begin ;binary output:
              WRITEU,lun,save_browser_data
              OPENW,hdrlun,varName+'.hdr',/GET
              PRINTF,hdrlun,'BINARY_DATA_FILE = "'+varName+extension+'"'
              PRINTF,hdrlun,'TOTAL_N_ELEMENTS = '+STRTRIM(dataInfo.n_elements,2)
              PRINTF,hdrlun,'N_DIMENSIONS = '+STRTRIM(dataInfo.n_dimensions,2)
              PRINTF,hdrlun,'DIMENSIONS = '+STRING(STRTRIM(dataInfo.dimensions,2),$
                FORMAT='(8(A,:," x "))')
              PRINTF,hdrlun,'DATA_TYPE = '+dataInfo.type_name
              PRINTF,hdrlun,'OFFSET = 0'
              if (BYTE(255,1) EQ 255) then byteOrd='MSB' else byteOrd='LSB'
              PRINTF,hdrlun,'BYTE_ORDER = '+byteOrd
              if (dataInfo.Type EQ 7) then PRINTF,hdrlun,'STRING_N_BYTES = '+STRTRIM(STRLEN(save_browser_data),2)
              FREE_LUN,hdrlun
            endelse
          end
        1 : $
          begin ;1-dimensional array:
            if (binaryFlag EQ 0) then begin ;ASCII output:
              ;convert byte data to integer so it doesn't get output as ASCII characters instead of numbers:
              if (dataInfo.Type EQ 1) then save_browser_data=FIX(TEMPORARY(save_browser_data))
              ;deal with double-precision, complex, and dcomplex data types:
              if (dataInfo.Type EQ 5) or (dataInfo.Type EQ 6) or (dataInfo.Type EQ 9) then begin
                if (dataInfo.Type EQ 5) then begin ;double:
                  maxData=STRTRIM(MAX(save_browser_data,MIN=minData),2)
                  maxData=STRMID(maxData,0,STRPOS(maxData,'.')+1)
                  decimals=14-STRLEN(maxData)
                  if (minData LT 0) then decimals=decimals-1
                  for k=0,dataInfo.dimensions[0]-2 do $
                    PRINTF,lun,STRTRIM(STRING(save_browser_data[k],FORMAT='(D14.'+STRTRIM(decimals,2)+')'),2)+separator
                  PRINTF,lun,STRTRIM(STRING(save_browser_data[dataInfo.dimensions[0]-1],FORMAT='(D14.'+STRTRIM(decimals,2)+')'),2)
                endif
                if (dataInfo.Type EQ 6) then begin ;complex:
                  for k=0,dataInfo.dimensions[0]-2 do $
                    PRINTF,lun,'('+STRTRIM(FLOAT(save_browser_data[k]),2)+', '+STRTRIM(IMAGINARY(save_browser_data[k]),2)+')'+separator
                  PRINTF,lun,'('+STRTRIM(FLOAT(save_browser_data[dataInfo.dimensions[0]-1]),2)+', '+STRTRIM(IMAGINARY(save_browser_data[dataInfo.dimensions[0]-1]),2)+')'
                endif
                if (dataInfo.Type EQ 9) then begin ;dcomplex:
                  maxReal=MAX(DOUBLE(save_browser_data),MIN=minReal)
                  maxImag=MAX(IMAGINARY(save_browser_data),MIN=minImag)
                  maxData=maxReal>maxImag
                  minData=minReal<minImag
                  maxData=STRTRIM(maxData,2)
                  maxData=STRMID(maxData,0,STRPOS(maxData,'.')+1)
                  decimals=14-STRLEN(maxData)
                  if (minData LT 0) then decimals=decimals-1
                  for k=0,dataInfo.dimensions[0]-2 do $
                    PRINTF,lun,'('+STRTRIM(STRING(DOUBLE(save_browser_data[k]),FORMAT='(D14.'+STRTRIM(decimals,2)+')'),2)+$
                      ', '+STRTRIM(STRING(IMAGINARY(save_browser_data[k]),FORMAT='(D14.'+STRTRIM(decimals,2)+')'),2)+')'+$
                      separator
                  PRINTF,lun,'('+STRTRIM(STRING(DOUBLE(save_browser_data[dataInfo.dimensions[0]-1]),$
                    FORMAT='(D14.'+STRTRIM(decimals,2)+')'),2)+$
                    ', '+STRTRIM(STRING(IMAGINARY(save_browser_data[dataInfo.dimensions[0]-1]),$
                    FORMAT='(D14.'+STRTRIM(decimals,2)+')'),2)+')'
                endif
              endif else begin ;everything else:
                for k=0,dataInfo.dimensions[0]-2 do $
                  PRINTF,lun,STRTRIM(save_browser_data[k],2)+separator
                PRINTF,lun,STRTRIM(save_browser_data[dataInfo.dimensions[0]-1],2)
              endelse
            endif else begin ;binary output:
              WRITEU,lun,save_browser_data
              OPENW,hdrlun,varName+'.hdr',/GET
              PRINTF,hdrlun,'BINARY_DATA_FILE = "'+varName+extension+'"'
              PRINTF,hdrlun,'TOTAL_N_ELEMENTS = '+STRTRIM(dataInfo.n_elements,2)
              PRINTF,hdrlun,'N_DIMENSIONS = '+STRTRIM(dataInfo.n_dimensions,2)
              PRINTF,hdrlun,'DIMENSIONS = '+STRING(STRTRIM(dataInfo.dimensions,2),$
                FORMAT='(8(A,:," x "))')
              PRINTF,hdrlun,'DATA_TYPE = '+dataInfo.type_name
              PRINTF,hdrlun,'OFFSET = 0'
              if (BYTE(255,1) EQ 255) then byteOrd='MSB' else byteOrd='LSB'
              PRINTF,hdrlun,'BYTE_ORDER = '+byteOrd
              if (dataInfo.Type EQ 7) then PRINTF,hdrlun,'STRING_N_BYTES = '+STRTRIM(ULONG64(TOTAL(STRLEN(save_browser_data))),2)
              FREE_LUN,hdrlun
            endelse
          end
        2 : $
          begin ;2-dimensional array:
            if (binaryFlag EQ 0) then begin ;ASCII output:
              ;convert byte data to integer so it doesn't get output as ASCII characters instead of numbers:
              if (dataInfo.Type EQ 1) then save_browser_data=FIX(TEMPORARY(save_browser_data))
              ;deal with floating-point, double-precision, complex, and dcomplex data types:
              if (dataInfo.Type EQ 4) or (dataInfo.Type EQ 5) or (dataInfo.Type EQ 6) or (dataInfo.Type EQ 9) then begin
                if (dataInfo.Type EQ 4) then begin ;float:
                  maxData=STRTRIM(MAX(save_browser_data,MIN=minData),2)
                  maxData=STRMID(maxData,0,STRPOS(maxData,'.')+1)
                  decimals=7-STRLEN(maxData)
                  if (minData LT 0) then decimals=decimals-1
                  if (j EQ 1) then begin ;space delimiter:
                    for k=0,dataInfo.dimensions[1]-1 do $
                      PRINTF,lun,STRING(save_browser_data[*,k],FORMAT='('+STRTRIM(dataInfo.dimensions[0],2)+$
                        '(F7.'+STRTRIM(decimals,2)+',:,"'+separator+'"))')
                  endif else begin ;tab or comma delimiter:
                    for k=0,dataInfo.dimensions[1]-1 do $
                      PRINTF,lun,STRING(STRTRIM(STRING(save_browser_data[*,k],FORMAT='(F7.'+STRTRIM(decimals,2)+')'),2),$
                        FORMAT='('+STRTRIM(dataInfo.dimensions[0],2)+'(A,:,"'+separator+'"))')
                  endelse
                endif
                if (dataInfo.Type EQ 5) then begin ;double:
                  maxData=STRTRIM(MAX(save_browser_data,MIN=minData),2)
                  maxData=STRMID(maxData,0,STRPOS(maxData,'.')+1)
                  decimals=14-STRLEN(maxData)
                  if (minData LT 0) then decimals=decimals-1
                  if (j EQ 1) then begin ;space delimiter:
                    for k=0,dataInfo.dimensions[1]-1 do $
                      PRINTF,lun,STRING(save_browser_data[*,k],FORMAT='('+STRTRIM(dataInfo.dimensions[0],2)+$
                        '(D14.'+STRTRIM(decimals,2)+',:,"'+separator+'"))')
                  endif else begin ;tab or comma delimiter:
                    for k=0,dataInfo.dimensions[1]-1 do $
                      PRINTF,lun,STRING(STRTRIM(STRING(save_browser_data[*,k],FORMAT='(D14.'+STRTRIM(decimals,2)+')'),2),$
                        FORMAT='('+STRTRIM(dataInfo.dimensions[0],2)+'(A,:,"'+separator+'"))')
                  endelse
                endif
                if (dataInfo.Type EQ 6) then begin ;complex:
                  maxReal=MAX(FLOAT(save_browser_data),MIN=minReal)
                  maxImag=MAX(IMAGINARY(save_browser_data),MIN=minImag)
                  maxData=maxReal>maxImag
                  minData=minReal<minImag
                  maxData=STRTRIM(maxData,2)
                  maxData=STRMID(maxData,0,STRPOS(maxData,'.')+1)
                  decimals=7-STRLEN(maxData)
                  if (minData LT 0) then decimals=decimals-1
                  for k=0,dataInfo.dimensions[1]-1 do begin
                    text=''
                    for l=0,dataInfo.dimensions[0]-2 do begin
                      text=text+'('+STRING(FLOAT(save_browser_data[l,k]),FORMAT='(F7.'+STRTRIM(decimals,2)+')')+', '+$
                        STRING(IMAGINARY(save_browser_data[l,k]),FORMAT='(F7.'+STRTRIM(decimals,2)+')')+')'+separator
                    endfor
                    text=text+'('+STRING(FLOAT(save_browser_data[dataInfo.dimensions[0]-1,k]),FORMAT='(F7.'+STRTRIM(decimals,2)+')')+', '+$
                      STRING(IMAGINARY(save_browser_data[dataInfo.dimensions[0]-1,k]),FORMAT='(F7.'+STRTRIM(decimals,2)+')')+')'
                    PRINTF,lun,TEMPORARY(text)
                  endfor
                endif
                if (dataInfo.Type EQ 9) then begin ;dcomplex:
                  maxReal=MAX(DOUBLE(save_browser_data),MIN=minReal)
                  maxImag=MAX(IMAGINARY(save_browser_data),MIN=minImag)
                  maxData=maxReal>maxImag
                  minData=minReal<minImag
                  maxData=STRTRIM(maxData,2)
                  maxData=STRMID(maxData,0,STRPOS(maxData,'.')+1)
                  decimals=14-STRLEN(maxData)
                  if (minData LT 0) then decimals=decimals-1
                  for k=0,dataInfo.dimensions[1]-1 do begin
                    text=''
                    for l=0,dataInfo.dimensions[0]-2 do begin
                      text=text+'('+STRING(DOUBLE(save_browser_data[l,k]),FORMAT='(D14.'+STRTRIM(decimals,2)+')')+', '+$
                        STRING(IMAGINARY(save_browser_data[l,k]),FORMAT='(D14.'+STRTRIM(decimals,2)+')')+')'+separator
                    endfor
                    text=text+'('+STRING(DOUBLE(save_browser_data[dataInfo.dimensions[0]-1,k]),FORMAT='(D14.'+STRTRIM(decimals,2)+')')+', '+$
                      STRING(IMAGINARY(save_browser_data[dataInfo.dimensions[0]-1,k]),FORMAT='(D14.'+STRTRIM(decimals,2)+')')+')'
                    PRINTF,lun,TEMPORARY(text)
                  endfor
                endif
              endif else begin ;everything else:
                ;deal with string arrays:
                if (dataInfo.Type EQ 7) then begin
                  for k=0,dataInfo.dimensions[1]-1 do $
                    PRINTF,lun,STRING(save_browser_data[*,k],$
                      FORMAT='('+STRTRIM(dataInfo.dimensions[0],2)+'(A,:,"'+separator+'"))')
                endif else begin
                  if (j EQ 1) then begin ;space delimiter:
                    length=STRTRIM(STRLEN(STRTRIM(MAX(save_browser_data),2)),2)
                    for k=0,dataInfo.dimensions[1]-1 do $
                      PRINTF,lun,STRING(STRTRIM(save_browser_data[*,k],2),$
                        FORMAT='('+STRTRIM(dataInfo.dimensions[0],2)+'(A'+length+',:,"'+separator+'"))')
                  endif else begin ;tab or comma delimiter:
                    for k=0,dataInfo.dimensions[1]-1 do $
                      PRINTF,lun,STRING(STRTRIM(save_browser_data[*,k],2),$
                        FORMAT='('+STRTRIM(dataInfo.dimensions[0],2)+'(A,:,"'+separator+'"))')
                  endelse
                endelse
              endelse
            endif else begin ;binary output:
              WRITEU,lun,save_browser_data
              OPENW,hdrlun,varName+'.hdr',/GET
              PRINTF,hdrlun,'BINARY_DATA_FILE = "'+varName+extension+'"'
              PRINTF,hdrlun,'TOTAL_N_ELEMENTS = '+STRTRIM(dataInfo.n_elements,2)
              PRINTF,hdrlun,'N_DIMENSIONS = '+STRTRIM(dataInfo.n_dimensions,2)
              PRINTF,hdrlun,'DIMENSIONS = '+STRING(STRTRIM(dataInfo.dimensions,2),$
                FORMAT='(8(A,:," x "))')
              PRINTF,hdrlun,'DATA_TYPE = '+dataInfo.type_name
              PRINTF,hdrlun,'OFFSET = 0'
              if (BYTE(255,1) EQ 255) then byteOrd='MSB' else byteOrd='LSB'
              PRINTF,hdrlun,'BYTE_ORDER = '+byteOrd
              if (dataInfo.Type EQ 7) then PRINTF,hdrlun,'STRING_N_BYTES = '+STRTRIM(ULONG64(TOTAL(STRLEN(save_browser_data))),2)
              FREE_LUN,hdrlun
            endelse
          end
        3 : $
          begin ;3-dimensional array:
            if (binaryFlag EQ 0) then begin ;ASCII output:
              ;convert byte data to integer so it doesn't get output as ASCII characters instead of numbers:
              if (dataInfo.Type EQ 1) then save_browser_data=FIX(TEMPORARY(save_browser_data))
              ;deal with floating-point, double-precision, complex, and dcomplex data types:
              if (dataInfo.Type EQ 4) or (dataInfo.Type EQ 5) or (dataInfo.Type EQ 6) or (dataInfo.Type EQ 9) then begin
                if (dataInfo.Type EQ 4) then begin ;float:
                  maxData=STRTRIM(MAX(save_browser_data,MIN=minData),2)
                  maxData=STRMID(maxData,0,STRPOS(maxData,'.')+1)
                  decimals=7-STRLEN(maxData)
                  if (minData LT 0) then decimals=decimals-1
                  if (j EQ 1) then begin ;space delimiter:
                    for l=0,dataInfo.dimensions[2]-1 do begin
                      for k=0,dataInfo.dimensions[1]-1 do $
                        PRINTF,lun,STRING(save_browser_data[*,k,l],FORMAT='('+STRTRIM(dataInfo.dimensions[0],2)+$
                          '(F7.'+STRTRIM(decimals,2)+',:,"'+separator+'"))')
                      PRINTF,lun,''
                    endfor
                  endif else begin ;tab or comma delimiter:
                    for l=0,dataInfo.dimensions[2]-1 do begin
                      for k=0,dataInfo.dimensions[1]-1 do $
                        PRINTF,lun,STRING(STRTRIM(STRING(save_browser_data[*,k,l],FORMAT='(F7.'+STRTRIM(decimals,2)+')'),2),$
                          FORMAT='('+STRTRIM(dataInfo.dimensions[0],2)+'(A,:,"'+separator+'"))')
                      PRINTF,lun,''
                    endfor
                  endelse
                endif
                if (dataInfo.Type EQ 5) then begin ;double:
                  maxData=STRTRIM(MAX(save_browser_data,MIN=minData),2)
                  maxData=STRMID(maxData,0,STRPOS(maxData,'.')+1)
                  decimals=14-STRLEN(maxData)
                  if (minData LT 0) then decimals=decimals-1
                  if (j EQ 1) then begin ;space delimiter:
                    for l=0,dataInfo.dimensions[2]-1 do begin
                      for k=0,dataInfo.dimensions[1]-1 do $
                        PRINTF,lun,STRING(save_browser_data[*,k,l],FORMAT='('+STRTRIM(dataInfo.dimensions[0],2)+$
                          '(D14.'+STRTRIM(decimals,2)+',:,"'+separator+'"))')
                      PRINTF,lun,''
                    endfor
                  endif else begin ;tab or comma delimiter:
                    for l=0,dataInfo.dimensions[2]-1 do begin
                      for k=0,dataInfo.dimensions[1]-1 do $
                        PRINTF,lun,STRING(STRTRIM(STRING(save_browser_data[*,k,l],FORMAT='(D14.'+STRTRIM(decimals,2)+')'),2),$
                          FORMAT='('+STRTRIM(dataInfo.dimensions[0],2)+'(A,:,"'+separator+'"))')
                      PRINTF,lun,''
                    endfor
                  endelse
                endif
                if (dataInfo.Type EQ 6) then begin ;complex:
                  maxReal=MAX(FLOAT(save_browser_data),MIN=minReal)
                  maxImag=MAX(IMAGINARY(save_browser_data),MIN=minImag)
                  maxData=maxReal>maxImag
                  minData=minReal<minImag
                  maxData=STRTRIM(maxData,2)
                  maxData=STRMID(maxData,0,STRPOS(maxData,'.')+1)
                  decimals=7-STRLEN(maxData)
                  if (minData LT 0) then decimals=decimals-1
                  for m=0,dataInfo.dimensions[2]-1 do begin
                    for k=0,dataInfo.dimensions[1]-1 do begin
                      text=''
                      for l=0,dataInfo.dimensions[0]-2 do begin
                        text=text+'('+STRING(FLOAT(save_browser_data[l,k,m]),FORMAT='(F7.'+STRTRIM(decimals,2)+')')+', '+$
                          STRING(IMAGINARY(save_browser_data[l,k,m]),FORMAT='(F7.'+STRTRIM(decimals,2)+')')+')'+separator
                      endfor
                      text=text+'('+STRING(FLOAT(save_browser_data[dataInfo.dimensions[0]-1,k,m]),FORMAT='(F7.'+STRTRIM(decimals,2)+')')+', '+$
                        STRING(IMAGINARY(save_browser_data[dataInfo.dimensions[0]-1,k,m]),FORMAT='(F7.'+STRTRIM(decimals,2)+')')+')'
                      PRINTF,lun,TEMPORARY(text)
                    endfor
                    PRINTF,lun,''
                  endfor
                endif
                if (dataInfo.Type EQ 9) then begin ;dcomplex:
                  maxReal=MAX(DOUBLE(save_browser_data),MIN=minReal)
                  maxImag=MAX(IMAGINARY(save_browser_data),MIN=minImag)
                  maxData=maxReal>maxImag
                  minData=minReal<minImag
                  maxData=STRTRIM(maxData,2)
                  maxData=STRMID(maxData,0,STRPOS(maxData,'.')+1)
                  decimals=14-STRLEN(maxData)
                  if (minData LT 0) then decimals=decimals-1
                  for m=0,dataInfo.dimensions[2]-1 do begin
                    for k=0,dataInfo.dimensions[1]-1 do begin
                      text=''
                      for l=0,dataInfo.dimensions[0]-2 do begin
                        text=text+'('+STRING(DOUBLE(save_browser_data[l,k,m]),FORMAT='(D14.'+STRTRIM(decimals,2)+')')+', '+$
                          STRING(IMAGINARY(save_browser_data[l,k,m]),FORMAT='(D14.'+STRTRIM(decimals,2)+')')+')'+separator
                      endfor
                      text=text+'('+STRING(DOUBLE(save_browser_data[dataInfo.dimensions[0]-1,k,m]),FORMAT='(D14.'+STRTRIM(decimals,2)+')')+', '+$
                        STRING(IMAGINARY(save_browser_data[dataInfo.dimensions[0]-1,k,m]),FORMAT='(D14.'+STRTRIM(decimals,2)+')')+')'
                      PRINTF,lun,TEMPORARY(text)
                    endfor
                    PRINTF,lun,''
                  endfor
                endif
              endif else begin ;everything else:
                ;deal with string arrays:
                if (dataInfo.Type EQ 7) then begin
                  for l=0,dataInfo.dimensions[2]-1 do begin
                    for k=0,dataInfo.dimensions[1]-1 do $
                      PRINTF,lun,STRING(save_browser_data[*,k,l],$
                        FORMAT='('+STRTRIM(dataInfo.dimensions[0],2)+'(A,:,"'+separator+'"))')
                    PRINTF,lun,''
                  endfor
                endif else begin
                  if (j EQ 1) then begin ;space delimiter:
                    length=STRTRIM(STRLEN(STRTRIM(MAX(save_browser_data),2)),2)
                    for l=0,dataInfo.dimensions[2]-1 do begin
                      for k=0,dataInfo.dimensions[1]-1 do $
                        PRINTF,lun,STRING(STRTRIM(save_browser_data[*,k,l],2),$
                          FORMAT='('+STRTRIM(dataInfo.dimensions[0],2)+'(A'+length+',:,"'+separator+'"))')
                      PRINTF,lun,''
                    endfor
                  endif else begin ;tab or comma delimiter:
                    for l=0,dataInfo.dimensions[2]-1 do begin
                      for k=0,dataInfo.dimensions[1]-1 do $
                        PRINTF,lun,STRING(STRTRIM(save_browser_data[*,k,l],2),$
                          FORMAT='('+STRTRIM(dataInfo.dimensions[0],2)+'(A,:,"'+separator+'"))')
                      PRINTF,lun,''
                    endfor
                  endelse
                endelse
              endelse
            endif else begin ;binary output:
              WRITEU,lun,save_browser_data
              OPENW,hdrlun,varName+'.hdr',/GET
              PRINTF,hdrlun,'BINARY_DATA_FILE = "'+varName+extension+'"'
              PRINTF,hdrlun,'TOTAL_N_ELEMENTS = '+STRTRIM(dataInfo.n_elements,2)
              PRINTF,hdrlun,'N_DIMENSIONS = '+STRTRIM(dataInfo.n_dimensions,2)
              PRINTF,hdrlun,'DIMENSIONS = '+STRING(STRTRIM(dataInfo.dimensions,2),$
                FORMAT='(8(A,:," x "))')
              PRINTF,hdrlun,'INTERLEAVE = BSQ'
              PRINTF,hdrlun,'DATA_TYPE = '+dataInfo.type_name
              PRINTF,hdrlun,'OFFSET = 0'
              if (BYTE(255,1) EQ 255) then byteOrd='MSB' else byteOrd='LSB'
              PRINTF,hdrlun,'BYTE_ORDER = '+byteOrd
              if (dataInfo.Type EQ 7) then PRINTF,hdrlun,'STRING_N_BYTES = '+STRTRIM(ULONG64(TOTAL(STRLEN(save_browser_data))),2)
              FREE_LUN,hdrlun
            endelse
          end
        else : $
          begin ;4-dimensional array or greater ... simply dump the data into the file:
            if (binaryFlag EQ 0) then begin ;ASCII output:
              ;convert byte data to integer so it doesn't get output as ASCII characters instead of numbers:
              if (dataInfo.Type EQ 1) then save_browser_data=FIX(TEMPORARY(save_browser_data))
              PRINTF,lun,save_browser_data
            endif else begin ;binary output:
              WRITEU,lun,save_browser_data
              OPENW,hdrlun,varName+'.hdr',/GET
              PRINTF,hdrlun,'BINARY_DATA_FILE = "'+varName+extension+'"'
              PRINTF,hdrlun,'TOTAL_N_ELEMENTS = '+STRTRIM(dataInfo.n_elements,2)
              PRINTF,hdrlun,'N_DIMENSIONS = '+STRTRIM(dataInfo.n_dimensions,2)
              PRINTF,hdrlun,'DIMENSIONS = '+STRING(STRTRIM(dataInfo.dimensions,2),$
                FORMAT='(8(A,:," x "))')
              PRINTF,hdrlun,'DATA_TYPE = '+dataInfo.type_name
              PRINTF,hdrlun,'OFFSET = 0'
              if (BYTE(255,1) EQ 255) then byteOrd='MSB' else byteOrd='LSB'
              PRINTF,hdrlun,'BYTE_ORDER = '+byteOrd
              if (dataInfo.Type EQ 7) then PRINTF,hdrlun,'STRING_N_BYTES = '+STRTRIM(ULONG64(TOTAL(STRLEN(save_browser_data))),2)
              FREE_LUN,hdrlun
            endelse
          end
      endcase
    endif else begin ;STRUCTURE:
      returnFlag=0B
      if (dataInfo.n_elements GT 1) then begin ;structure array:
        ;first, close the currently open output filename because we need to output
        ;an individual file for each and every structure:
        FREE_LUN,TEMPORARY(lun)
        FILE_DELETE,varName+extension,/QUIET
        for s=0,dataInfo.n_elements-1 do begin
          ;open the default output filename for writing:
          OPENW,lun,varName+STRTRIM(s+1,2)+extension,/GET
          currData=save_browser_data[s]
          currDataInfo=SIZE(currData,/STRUCTURE)
          if (binaryFlag EQ 0) then begin ;ASCII output:
            ;convert byte data to integer so it doesn't get output as ASCII characters instead of numbers:
            if (currDataInfo.Type EQ 1) then currData=FIX(TEMPORARY(currData))
            ;loop through the tags and output the data:
            tagNames=TAG_NAMES(currData)
            for k=0,N_TAGS(currData)-1 do begin
              if (SIZE(currData.(k),/TYPE) EQ 10) then begin ;pointer:
                if (structPtrFlag EQ 0) then begin
                  void=DIALOG_MESSAGE(['The following structure variable contains pointers :',$
                                       '','"'+varName+'"','',$
                                       'In order to output the heap variable data referenced by these pointers',$
                                       'you must expand the structure tree and select the individual structure',$
                                       'tags for the desired variables.'])
                  structPtrFlag=1B
                endif
              endif
              if (SIZE(currData.(k),/TYPE) EQ 11) then begin ;object:
                if (structObjFlag EQ 0) then begin
                  void=DIALOG_MESSAGE(['The following structure variable contains object references :',$
                                       '','"'+varName+'"','',$
                                       'The output of instance data associated with an object is not supported',$
                                       'by this program.  In order to access the data associated with these',$
                                       'objects you must restore the selected save file into the IDL software',$
                                       'package and obtain the data using the appropriate methods.'])
                  structObjFlag=1B
                endif
              endif
              PRINTF,lun,'********************************************************************************'
              PRINTF,lun,'Tag #'+STRTRIM(k+1,2)+' [Name = "'+tagNames[k]+'"]'
              PRINTF,lun,currData.(k)
              PRINTF,lun,'********************************************************************************'
              PRINTF,lun,''
              PRINTF,lun,''
            endfor
          endif else begin ;binary output:
            for k=0,N_TAGS(currData)-1 do begin
              if (SIZE(currData.(k),/TYPE) EQ 10) or (SIZE(currData.(k),/TYPE) EQ 11) then begin
                if (structBinFlag EQ 0) then begin
                  void=DIALOG_MESSAGE(['The following structure variable contains pointers and/or objects :',$
                                       '','"'+varName+'"','',$
                                       'The output of entire structures that contain pointers and/or object',$
                                       'references to the Structured Binary (*.bin) output file format is not',$
                                       'supported by this program.  In order to output the heap variable',$
                                       'data referenced by any pointers you must expand the structure tree',$
                                       'and select the individual structure tags for the desired variables.'])
                  structBinFlag=1B
                endif
                returnFlag=1B
              endif
            endfor
            if (returnFlag EQ 1) then begin
              if (s LT (dataInfo.n_elements-1)) then FREE_LUN,TEMPORARY(lun)
              FILE_DELETE,varName+STRTRIM(s+1,2)+extension,/QUIET
              ;cleanup heap memory:
              HEAP_FREE,save_browser_data
              ;re-create IDL_SAVEFILE object so heap variable data can be accessed in the future:
              OBJ_DESTROY,(*(*pState).oSaveFiles)[saveIndex]
              (*(*pState).oSaveFiles)[saveIndex]=OBJ_NEW('IDL_SAVEFILE',(*pState).files[saveIndex],$
                                                         /RELAXED_STRUCTURE_ASSIGNMENT)
              RETURN
            endif
            HELP,currData,/STRUCTURE,OUTPUT=structText
            WRITEU,lun,TEMPORARY(currData)
            OPENW,hdrlun,varName+STRTRIM(s+1,2)+'.hdr',/GET
            PRINTF,hdrlun,'BINARY_DATA_FILE = "'+varName+extension+'"'
            PRINTF,hdrlun,'TOTAL_N_ELEMENTS = '+STRTRIM(currDataInfo.n_elements,2)
            PRINTF,hdrlun,'N_DIMENSIONS = '+STRTRIM(currDataInfo.n_dimensions,2)
            PRINTF,hdrlun,'DIMENSIONS = '+STRING(STRTRIM(currDataInfo.dimensions,2),$
              FORMAT='(8(A,:," x "))')
            PRINTF,hdrlun,'DATA_TYPE = '+currDataInfo.type_name
            PRINTF,hdrlun,'OFFSET = 0'
            if (BYTE(255,1) EQ 255) then byteOrd='MSB' else byteOrd='LSB'
            PRINTF,hdrlun,'BYTE_ORDER = '+byteOrd
            PRINTF,hdrlun,'STRUCTURE_LAYOUT = '
            for k=1,N_ELEMENTS(structText)-1 do PRINTF,hdrlun,structText[k]
            FREE_LUN,hdrlun
          endelse
          ;close the file:
          if (s LT (dataInfo.n_elements-1)) then FREE_LUN,TEMPORARY(lun)
        endfor
      endif else begin ;scalar structure:
        if (binaryFlag EQ 0) then begin ;ASCII output:
          ;convert byte data to integer so it doesn't get output as ASCII characters instead of numbers:
          if (dataInfo.Type EQ 1) then save_browser_data=FIX(TEMPORARY(save_browser_data))
          ;loop through the tags and output the data:
          tagNames=TAG_NAMES(save_browser_data)
          for k=0,N_TAGS(save_browser_data)-1 do begin
            if (SIZE(save_browser_data.(k),/TYPE) EQ 10) then begin
              if (structPtrFlag EQ 0) then begin ;pointer:
                void=DIALOG_MESSAGE(['The following structure variable contains pointers :',$
                                     '','"'+varName+'"','',$
                                     'In order to output the heap variable data referenced by these pointers',$
                                     'you must expand the structure tree and select the individual structure',$
                                     'tags for the desired variables.'])
                structPtrFlag=1B
              endif
            endif
            if (SIZE(save_browser_data.(k),/TYPE) EQ 11) then begin ;object:
              if (structObjFlag EQ 0) then begin
                void=DIALOG_MESSAGE(['The following structure variable contains object references :',$
                                     '','"'+varName+'"','',$
                                     'The output of instance data associated with an object is not supported',$
                                     'by this program.  In order to access the data associated with these',$
                                     'objects you must restore the selected save file into the IDL software',$
                                     'package and obtain the data using the appropriate methods.'])
                structObjFlag=1B
              endif
            endif
            PRINTF,lun,'********************************************************************************'
            PRINTF,lun,'Tag #'+STRTRIM(k+1,2)+' [Name = "'+tagNames[k]+'"]'
            PRINTF,lun,save_browser_data.(k)
            PRINTF,lun,'********************************************************************************'
            PRINTF,lun,''
            PRINTF,lun,''
          endfor
        endif else begin ;binary output:
          for k=0,N_TAGS(save_browser_data)-1 do begin
            if (SIZE(save_browser_data.(k),/TYPE) EQ 10) or (SIZE(save_browser_data.(k),/TYPE) EQ 11) then begin
              if (structBinFlag EQ 0) then begin
                void=DIALOG_MESSAGE(['The following structure variable contains pointers and/or objects :',$
                                     '','"'+varName+'"','',$
                                     'The output of entire structures that contain pointers and/or object',$
                                     'references to the Structured Binary (*.bin) output file format is not',$
                                     'supported by this program.  In order to output the heap variable',$
                                     'data referenced by any pointers you must expand the structure tree',$
                                     'and select the individual structure tags for the desired variables.'])
                structBinFlag=1B
              endif
              returnFlag=1B
            endif
          endfor
          if (returnFlag EQ 1) then begin
            WIDGET_CONTROL,TEMPORARY(tlb),/DESTROY
            FREE_LUN,TEMPORARY(lun)
            FILE_DELETE,varName+extension,/QUIET
            WIDGET_CONTROL,saveID,SENSITIVE=0
            ;cleanup heap memory:
            HEAP_FREE,save_browser_data
            ;re-create IDL_SAVEFILE object so heap variable data can be accessed in the future:
            OBJ_DESTROY,(*(*pState).oSaveFiles)[saveIndex]
            (*(*pState).oSaveFiles)[saveIndex]=OBJ_NEW('IDL_SAVEFILE',(*pState).files[saveIndex],$
                                                       /RELAXED_STRUCTURE_ASSIGNMENT)
            RETURN
          endif
          HELP,save_browser_data,/STRUCTURE,OUTPUT=structText
          WRITEU,lun,save_browser_data
          OPENW,hdrlun,varName+'.hdr',/GET
          PRINTF,hdrlun,'BINARY_DATA_FILE = "'+varName+extension+'"'
          PRINTF,hdrlun,'TOTAL_N_ELEMENTS = '+STRTRIM(dataInfo.n_elements,2)
          PRINTF,hdrlun,'N_DIMENSIONS = '+STRTRIM(dataInfo.n_dimensions,2)
          PRINTF,hdrlun,'DIMENSIONS = '+STRING(STRTRIM(dataInfo.dimensions,2),$
            FORMAT='(8(A,:," x "))')
          PRINTF,hdrlun,'DATA_TYPE = '+dataInfo.type_name
          PRINTF,hdrlun,'OFFSET = 0'
          if (BYTE(255,1) EQ 255) then byteOrd='MSB' else byteOrd='LSB'
          PRINTF,hdrlun,'BYTE_ORDER = '+byteOrd
          PRINTF,hdrlun,'STRUCTURE_LAYOUT = '
          for k=1,N_ELEMENTS(structText)-1 do PRINTF,hdrlun,structText[k]
          FREE_LUN,hdrlun
        endelse
      endelse
      ;cleanup heap memory:
      HEAP_FREE,save_browser_data
      ;re-create IDL_SAVEFILE object so heap variable data can be accessed in the future:
      OBJ_DESTROY,(*(*pState).oSaveFiles)[saveIndex]
      (*(*pState).oSaveFiles)[saveIndex]=OBJ_NEW('IDL_SAVEFILE',(*pState).files[saveIndex],$
                                                 /RELAXED_STRUCTURE_ASSIGNMENT)
    endelse
    FREE_LUN,TEMPORARY(lun)
  endfor
ENDFOR
WIDGET_CONTROL,saveID,SENSITIVE=0
WIDGET_CONTROL,TEMPORARY(tlb),/DESTROY
END
;****************************************************************************************************


;****************************************************************************************************
;+
; <PRO SAVE_BROWSER_CLEANUP>
; General description of routine
;
; @Param
;   widgetID {in|out} {optional|required} {type=} {default=}
;     No Description
;
; @Author
;   Adam O'Connor
;
; @Copyright
;   Copyright &#169; 2006 ITT Industries. All rights reserved.
;
; @Hidden
;-
PRO SAVE_BROWSER_CLEANUP,widgetID
COMPILE_OPT idl2,hidden
;error handling ... reset !ERROR_STATE:
MESSAGE,/RESET
;catch error:
CATCH,error
if (error NE 0) then begin ;error caught:
  ;obtain last message:
  HELP,/LAST_MESSAGE,OUTPUT=traceback
  messageStr=['Error Caught :','',traceback]
  ;display in dialog:
  void=DIALOG_MESSAGE(messageStr,/ERROR)
  ;terminate execution of this program:
  RETURN
endif
;obtain state structure for top-level-base from its uvalue:
WIDGET_CONTROL,widgetID,GET_UVALUE=pState
;test for validity of state structure pointer:
if PTR_VALID(pState) then begin
  ;cleanup heap memory:
  for s=0,N_ELEMENTS(*(*pState).oSaveFiles)-1 do begin
    if OBJ_VALID((*(*pState).oSaveFiles)[s]) then OBJ_DESTROY,(*(*pState).oSaveFiles)[s]
  endfor
  PTR_FREE,(*pState).oSaveFiles
  PTR_FREE,(*pState).dataTreeIDs
  ;reset original settings:
  !QUIET=(*pState).quietInit
  SET_PLOT,(*pState).currDevice
  CD,(*pState).currentDir
  ;display message if necessary:
  if ((*pState).resetSessionWarnFlag EQ 1) and ((*pState).restoreFlag EQ 1) then $
    void=DIALOG_MESSAGE(['During execution of this program the contents of one or more of the selected save',$
                         'files was restored into the current IDL session in order to determine the parameters',$
                         'for routines contained within the file.  These routines will remain compiled in memory',$
                         'until the session is reset or IDL is closed and restarted.'], /INFORMATION)
  PTR_FREE,TEMPORARY(pState)
endif
END
;****************************************************************************************************


;****************************************************************************************************
;+
; <PRO SAVE_BROWSER_CANCEL>
; General description of routine
;
; @Param
;   event {in|out} {optional|required} {type=} {default=}
;     No Description
;
; @Author
;   Adam O'Connor
;
; @Copyright
;   Copyright &#169; 2006 ITT Industries. All rights reserved.
;
; @Hidden
;-
PRO SAVE_BROWSER_CANCEL,event
COMPILE_OPT idl2,hidden
;terminate the program by destroying the top-level-base (widgetID always stored in event.top):
WIDGET_CONTROL,event.top,/DESTROY
END
;****************************************************************************************************


;****************************************************************************************************
;+
; <FUNCTION SAVE_BROWSER_CREATE_TREE>
; General description of routine
;
; @Param
;   tlb {in|out} {optional|required} {type=} {default=}
;     No Description
;
; @Param
;   treeBase {in|out} {optional|required} {type=} {default=}
;     No Description
;
; @Param
;   files {in|out} {optional|required} {type=} {default=}
;     No Description
;
; @Param
;   oSaveFiles {in|out} {optional|required} {type=} {default=}
;     No Description
;
; @Param
;   saveContents {in|out} {optional|required} {type=} {default=}
;     No Description
;
; @Keyword
;   GET_DATATREEIDS {in|out} {optional|required} {type=} {default=}
;     No Description
;
; @Keyword
;   BITMAPS {in|out} {optional|required} {type=} {default=}
;     No Description
;
; @Keyword
;   SCREEN_SIZE {in|out} {optional|required} {type=} {default=}
;     No Description
;
; @Keyword
;   RESET_SESSION_WARN {in|out} {optional|required} {type=} {default=}
;     No Description
;
; @Author
;   Adam O'Connor
;
; @Copyright
;   Copyright &#169; 2006 ITT Industries. All rights reserved.
;
; @Hidden
;
; @Returns
;   Result
;-
FUNCTION SAVE_BROWSER_CREATE_TREE,tlb,treeBase,files,oSaveFiles,saveContents,$
  GET_DATATREEIDS=dataTreeIDs,BITMAPS=bitmaps,SCREEN_SIZE=screenSize,RESET_SESSION_WARN=resetSessionWarnFlag
COMPILE_OPT idl2,hidden
;error handling ... reset !ERROR_STATE:
MESSAGE,/RESET
;catch error:
CATCH,error
if (error NE 0) then begin ;error caught:
  case (!ERROR_STATE.NAME) of
    'IDL_M_SR_MISTAGTYP' : $
      begin
        if (structDefWarnFlag EQ 0) then begin
          void=DIALOG_MESSAGE(['One or more of the selected save files contain data variables that have structure',$
                               'definitions which are incompatible with the current session of IDL.  This usually',$
                               'occurs when an object is saved to a save file in an older version of IDL and the',$
                               'class structure has changed in more recent versions of the software.  In order to',$
                               'access the structures with conflicting definitions you must restore these save files',$
                               'into the IDL software package using the relaxed structure assignment functionality.'])
          structDefWarnFlag=1B
        endif
        structDefFlag=1B
        if (varType EQ 'STRUCT') then GOTO,structTryAgain
        if (varType EQ 'OBJREF') then GOTO,objectTryAgain
      end
    'IDL_M_SR_STRABORT' : $
      begin
        if (structDefWarnFlag EQ 0) then begin
          void=DIALOG_MESSAGE(['One or more of the selected save files contain data variables that have structure',$
                               'definitions which are incompatible with the current session of IDL.  This usually',$
                               'occurs when an object is saved to a save file in an older version of IDL and the',$
                               'class structure has changed in more recent versions of the software.  In order to',$
                               'access the structures with conflicting definitions you must restore these save files',$
                               'into the IDL software package using the relaxed structure assignment functionality.'])
          structDefWarnFlag=1B
        endif
        structDefFlag=1B
        if (varType EQ 'STRUCT') then GOTO,structTryAgain
        if (varType EQ 'OBJREF') then GOTO,objectTryAgain
      end
    'IDL_M_SR_PRO_BADVER_NAME' : $
      begin
        MESSAGE,/RESET
        paramsFlag=0B
        if (oldSaveWarnFlag EQ 0) then begin
          void=DIALOG_MESSAGE(['One or more of the selected save files contain routines that were compiled and',$
                               'saved in an older verson of IDL which had an internal pcode level that is',$
                               'incompatible with the current session of IDL.  Consequently, the arguments and',$
                               'keywords for the routines contained in these save files cannot be determined.'])
          oldSaveWarnFlag=1B
        endif
        GOTO,routineTryAgain
      end
    else : $
      begin
        ;obtain last message:
        HELP,/LAST_MESSAGE,OUTPUT=traceback
        messageStr=['Error Caught :','',traceback]
        ;display in dialog:
        void=DIALOG_MESSAGE(messageStr,/ERROR)
        ;terminate execution of this program:
        RETURN,-1
      end
  endcase
endif
;obtain bitmaps from state structure if they are not passed in using the BITMAPS keyword:
if (N_ELEMENTS(bitmaps) EQ 0) then begin ;program is already running:
  WIDGET_CONTROL,tlb,GET_UVALUE=pState
  bitmaps=(*pState).bitmaps
endif
;obtain screen size from state structure if it is not passed in using the SCREEN_SIZE keyword:
if (N_ELEMENTS(screenSize) EQ 0) then begin ;program is already running:
  WIDGET_CONTROL,tlb,GET_UVALUE=pState
  screenSize=(*pState).screenSize
endif
dataTreeIDs=[0L]
if ((screenSize[0] * screenSize[1]) LT 786432) or (screenSize[1] LT 768) then begin
  xSize=740
  ySize=460
endif else begin
  xSize=900
  ySize=625
endelse
tree=WIDGET_TREE(treeBase,/MULTIPLE,XSIZE=xSize,YSIZE=ySize,UNAME='TREE')
WAIT,0.1
oldSaveWarnFlag=0B
structDefWarnFlag=0B
resetSessionWarnFlag=0B
for f=0,N_ELEMENTS(files)-1 do begin
  structDefFlag=0B
  structRestoreFlag=0B
  saveNode=WIDGET_TREE(tree,/FOLDER,/MULTIPLE,VALUE=saveContents[f].filename)
  infoNode=WIDGET_TREE(saveNode,/FOLDER,VALUE='INFORMATION')
  if (saveContents[f].description NE '') then $
    node=WIDGET_TREE(infoNode,VALUE='DESCRIPTION = "'+saveContents[f].description+'"')
  node=WIDGET_TREE(infoNode,VALUE='FILETYPE = "'+saveContents[f].filetype+'"')
  node=WIDGET_TREE(infoNode,VALUE='USER = "'+saveContents[f].user+'"')
  node=WIDGET_TREE(infoNode,VALUE='HOST = "'+saveContents[f].host+'"')
  node=WIDGET_TREE(infoNode,VALUE='DATE = "'+saveContents[f].date+'"')
  node=WIDGET_TREE(infoNode,VALUE='ARCH = "'+saveContents[f].arch+'"')
  node=WIDGET_TREE(infoNode,VALUE='OS = "'+saveContents[f].os+'"')
  node=WIDGET_TREE(infoNode,VALUE='RELEASE = "'+saveContents[f].release+'"')
  ;test to see if save file contains routines or data:
  if (saveContents[f].n_procedure NE 0) or (saveContents[f].n_function NE 0) then begin
    ;save file contains routines ... restore entire save file so parameters can be determined:
    RESTORE,files[f]
    resetSessionWarnFlag=1B
    paramsFlag=1B
    if (!ERROR_STATE.NAME EQ 'IDL_M_SR_PRO_BADVER_NAME') then begin ;error restoring old save file:
      MESSAGE,/RESET
      paramsFlag=0B
      if (oldSaveWarnFlag EQ 0) then begin
        void=DIALOG_MESSAGE(['One or more of the selected save files contain routines that were compiled and',$
                             'saved in an older verson of IDL which had an internal pcode level that is',$
                             'incompatible with the current session of IDL.  Consequently, the arguments and',$
                             'keywords for the routines contained in these save files cannot be determined.'])
        oldSaveWarnFlag=1B
      endif
    endif
    routineTryAgain:
    if (saveContents[f].n_procedure NE 0) then begin
      procedureNode=WIDGET_TREE(saveNode,/FOLDER,VALUE='COMPILED PROCEDURES')
      procedureNames=oSaveFiles[f]->NAMES(/PROCEDURE,COUNT=count)
      for g=0,count-1 do begin
        if (paramsFlag EQ 1) then begin
          params=ROUTINE_INFO(procedureNames[g],/PARAMETERS)
          if ((params.num_args+params.num_kw_args) GE 1) then folder=1 else folder=0
        endif else begin
          folder=0
        endelse
        routineNode=WIDGET_TREE(procedureNode,BITMAP=bitmaps[*,*,*,10],FOLDER=folder,VALUE=procedureNames[g])
        if (paramsFlag EQ 1) then begin
          if (params.num_args NE 0) then begin
            argNode=WIDGET_TREE(routineNode,/FOLDER,VALUE='ARGUMENTS')
            for h=0,params.num_args-1 do begin
              node=WIDGET_TREE(argNode,VALUE=params.args[h])
            endfor
          endif
          if (params.num_kw_args NE 0) then begin
            kwNode=WIDGET_TREE(routineNode,/FOLDER,VALUE='KEYWORDS')
            for h=0,params.num_kw_args-1 do begin
              node=WIDGET_TREE(kwNode,VALUE=params.kw_args[h])
            endfor
          endif
        endif
      endfor
    endif
    if (saveContents[f].n_function NE 0) then begin
      functionNode=WIDGET_TREE(saveNode,/FOLDER,VALUE='COMPILED FUNCTIONS')
      functionNames=oSaveFiles[f]->NAMES(/FUNCTIO,COUNT=count)
      for g=0,count-1 do begin
        if (paramsFlag EQ 1) then begin
          params=ROUTINE_INFO(functionNames[g],/PARAMETERS,/FUNCTIONS)
          if ((params.num_args+params.num_kw_args) GE 1) then folder=1 else folder=0
        endif else begin
          folder=0
        endelse
        routineNode=WIDGET_TREE(functionNode,BITMAP=bitmaps[*,*,*,10],FOLDER=folder,VALUE=functionNames[g])
        if (paramsFlag EQ 1) then begin
          if (params.num_args NE 0) then begin
            argNode=WIDGET_TREE(routineNode,/FOLDER,VALUE='ARGUMENTS')
            for h=0,params.num_args-1 do begin
              node=WIDGET_TREE(argNode,VALUE=params.args[h])
            endfor
          endif
          if (params.num_kw_args NE 0) then begin
            kwNode=WIDGET_TREE(routineNode,/FOLDER,VALUE='KEYWORDS')
            for h=0,params.num_kw_args-1 do begin
              node=WIDGET_TREE(kwNode,VALUE=params.kw_args[h])
            endfor
          endif
        endif
      endfor
    endif
  endif else begin ;save file contains data:
    if (saveContents[f].n_var NE 0) then begin
      varNode=WIDGET_TREE(saveNode,/FOLDER,/MULTIPLE,VALUE='DATA VARIABLES')
      varNames=oSaveFiles[f]->NAMES(COUNT=count)
      for g=0,count-1 do begin
        varType=oSaveFiles[f]->SIZE(varNames[g],/TNAME)
        varNDims=oSaveFiles[f]->SIZE(varNames[g],/N_DIMENSIONS)
        varDims=oSaveFiles[f]->SIZE(varNames[g],/DIMENSIONS)
        if (varNDims EQ 0) then $
          varText=varNames[g]+'          Type='+varType+'     Dimensions=SCALAR' $
        else $
          varText=varNames[g]+'          Type='+varType+'     Dimensions=ARRAY['+$
                  STRING(STRTRIM(varDims,2),FORMAT='(8(A,:,","))')+']'
        case varType of
          'STRING' : $
            begin ;string:
              if (varNDims EQ 0) then $
                node=WIDGET_TREE(varNode,BITMAP=bitmaps[*,*,*,1],VALUE=varText,UVALUE=STRTRIM(f,2)+varNames[g]) $
              else $
                node=WIDGET_TREE(varNode,BITMAP=bitmaps[*,*,*,2],VALUE=varText,UVALUE=STRTRIM(f,2)+varNames[g])
              dataTreeIDs=[dataTreeIDs,node]
            end
          'STRUCT' : $
            begin ;structure:
              ;attempt to restore the structure:
              structSize=oSaveFiles[f]->SIZE(varNames[g],/STRUCTURE)
              if (structSize.structure_name NE '') then $
                varText=varText+'     Name="'+structSize.structure_name+'"'
              structTryAgain:
              if (structDefFlag EQ 1) then begin
                structDefFlag=0B
                varText=varText+'     {This variable contains a structure with an incompatible definition and cannot be accessed}'
                structNode=WIDGET_TREE(varNode,BITMAP=bitmaps[*,*,*,5],VALUE=varText,SENSITIVE=0,$
                                       UVALUE=STRTRIM(f,2)+varNames[g])
              endif else begin
                oSaveFiles[f]->RESTORE,varNames[g]
                structRestoreFlag=1B
                save_browser_structure=(SCOPE_VARFETCH(varNames[g],LEVEL=0))
                HEAP_FREE,TEMPORARY((SCOPE_VARFETCH(varNames[g],LEVEL=0)))
                nTags=N_TAGS(save_browser_structure)
                tagNames=TAG_NAMES(save_browser_structure)
                structNode=WIDGET_TREE(varNode,/FOLDER,/MULTIPLE,BITMAP=bitmaps[*,*,*,5],VALUE=varText,$
                                       UVALUE=STRTRIM(f,2)+varNames[g])
                dataTreeIDs=[dataTreeIDs,structNode]
                for h=0,nTags-1 do begin
                  tagType=SIZE(save_browser_structure.(h),/TNAME)
                  tagNDims=SIZE(save_browser_structure.(h),/N_DIMENSIONS)
                  tagDims=SIZE(save_browser_structure.(h),/DIMENSIONS)
                  if (tagNDims EQ 0) then $
                    tagText=tagNames[h]+'          Type='+tagType+'     Dimensions=SCALAR' $
                  else $
                    tagText=tagNames[h]+'          Type='+tagType+'     Dimensions=ARRAY['+$
                            STRING(STRTRIM(tagDims,2),FORMAT='(8(A,:,","))')+']'
                  case tagType of
                    'STRING' : $
                      begin ;string:
                        if (tagNDims EQ 0) then $
                          node=WIDGET_TREE(structNode,BITMAP=bitmaps[*,*,*,1],VALUE=tagText,UVALUE=STRTRIM(f,2)+varNames[g]+'.'+tagNames[h]) $
                        else $
                          node=WIDGET_TREE(structNode,BITMAP=bitmaps[*,*,*,2],VALUE=tagText,UVALUE=STRTRIM(f,2)+varNames[g]+'.'+tagNames[h])
                        dataTreeIDs=[dataTreeIDs,node]
                      end
                    'STRUCT' : $
                      begin ;structure:
                        nSubTags=N_TAGS(save_browser_structure.(h))
                        tagSubNames=TAG_NAMES(save_browser_structure.(h))
                        structSubSize=SIZE(save_browser_structure.(h),/STRUCTURE)
                        if (structSubSize.structure_name NE '') then $
                          tagText=tagText+'     Name="'+structSubSize.structure_name+'"'
                        structSubNode=WIDGET_TREE(structNode,/FOLDER,/MULTIPLE,BITMAP=bitmaps[*,*,*,5],VALUE=tagText,$
                                                  UVALUE=STRTRIM(f,2)+varNames[g]+'.'+tagNames[h])
                        dataTreeIDs=[dataTreeIDs,structSubNode]
                        for i=0,nSubTags-1 do begin
                          tagSubType=SIZE(save_browser_structure.(h).(i),/TNAME)
                          tagSubNDims=SIZE(save_browser_structure.(h).(i),/N_DIMENSIONS)
                          tagSubDims=SIZE(save_browser_structure.(h).(i),/DIMENSIONS)
                          if (tagSubNDims EQ 0) then $
                            tagSubText=tagSubNames[i]+'          Type='+tagSubType+'     Dimensions=SCALAR' $
                          else $
                            tagSubText=tagSubNames[i]+'          Type='+tagSubType+'     Dimensions=ARRAY['+$
                                       STRING(STRTRIM(tagSubDims,2),FORMAT='(8(A,:,","))')+']'
                          case tagSubType of
                            'STRING' : $
                              begin ;string:
                                if (tagSubNDims EQ 0) then $
                                  node=WIDGET_TREE(structSubNode,BITMAP=bitmaps[*,*,*,1],VALUE=tagSubText,$
                                                   UVALUE=STRTRIM(f,2)+varNames[g]+'.'+tagNames[h]+'.'+tagSubNames[i]) $
                                else $
                                  node=WIDGET_TREE(structSubNode,BITMAP=bitmaps[*,*,*,2],VALUE=tagSubText,$
                                                   UVALUE=STRTRIM(f,2)+varNames[g]+'.'+tagNames[h]+'.'+tagSubNames[i])
                                dataTreeIDs=[dataTreeIDs,node]
                              end
                            'STRUCT' : $
                              begin ;structure:
                                structSubSubSize=SIZE(save_browser_structure.(h).(i),/STRUCTURE)
                                if (structSubSubSize.structure_name NE '') then $
                                  tagSubText=tagSubText+'     Name="'+structSubSubSize.structure_name+'"'
                                node=WIDGET_TREE(structSubNode,BITMAP=bitmaps[*,*,*,5],VALUE=tagSubText,$
                                                 UVALUE=STRTRIM(f,2)+varNames[g]+'.'+tagNames[h]+'.'+tagSubNames[i])
                                dataTreeIDs=[dataTreeIDs,node]
                              end
                            'POINTER' : $
                              begin ;pointer:
                                node=WIDGET_TREE(structSubNode,BITMAP=bitmaps[*,*,*,4],VALUE=tagSubText,$
                                                 UVALUE=STRTRIM(f,2)+varNames[g]+'.'+tagNames[h]+'.'+tagSubNames[i])
                                dataTreeIDs=[dataTreeIDs,node]
                              end
                            'OBJREF' : $
                              begin ;object:
                                node=WIDGET_TREE(structSubNode,BITMAP=bitmaps[*,*,*,6],VALUE=tagSubText,$
                                                 UVALUE=STRTRIM(f,2)+varNames[g]+'.'+tagNames[h]+'.'+tagSubNames[i])
                                dataTreeIDs=[dataTreeIDs,node]
                              end
                            else : $
                              begin ;normal:
                                if (tagSubNDims EQ 0) then $
                                  node=WIDGET_TREE(structSubNode,BITMAP=bitmaps[*,*,*,0],VALUE=tagSubText,$
                                                   UVALUE=STRTRIM(f,2)+varNames[g]+'.'+tagNames[h]+'.'+tagSubNames[i]) $
                                else $
                                  node=WIDGET_TREE(structSubNode,BITMAP=bitmaps[*,*,*,3],VALUE=tagSubText,$
                                                   UVALUE=STRTRIM(f,2)+varNames[g]+'.'+tagNames[h]+'.'+tagSubNames[i])
                                dataTreeIDs=[dataTreeIDs,node]
                              end
                            endcase
                        endfor
                      end
                    'POINTER' : $
                      begin ;pointer:
                        node=WIDGET_TREE(structNode,BITMAP=bitmaps[*,*,*,4],VALUE=tagText,UVALUE=STRTRIM(f,2)+varNames[g]+'.'+tagNames[h])
                        dataTreeIDs=[dataTreeIDs,node]
                      end
                    'OBJREF' : $
                      begin ;object:
                        node=WIDGET_TREE(structNode,BITMAP=bitmaps[*,*,*,6],VALUE=tagText,UVALUE=STRTRIM(f,2)+varNames[g]+'.'+tagNames[h])
                        dataTreeIDs=[dataTreeIDs,node]
                      end
                    else : $
                      begin ;normal:
                        if (tagNDims EQ 0) then $
                          node=WIDGET_TREE(structNode,BITMAP=bitmaps[*,*,*,0],VALUE=tagText,UVALUE=STRTRIM(f,2)+varNames[g]+'.'+tagNames[h]) $
                        else $
                          node=WIDGET_TREE(structNode,BITMAP=bitmaps[*,*,*,3],VALUE=tagText,UVALUE=STRTRIM(f,2)+varNames[g]+'.'+tagNames[h])
                        dataTreeIDs=[dataTreeIDs,node]
                      end
                  endcase
                endfor
              endelse
            end
          'POINTER' : $
            begin ;pointer:
              node=WIDGET_TREE(varNode,BITMAP=bitmaps[*,*,*,4],VALUE=varText,UVALUE=STRTRIM(f,2)+varNames[g])
              dataTreeIDs=[dataTreeIDs,node]
            end
          'OBJREF' : $
            begin ;object:
              objectTryAgain:
              if (structDefFlag EQ 1) then begin
                structDefFlag=0B
                varText=varText+'     {This variable contains a structure with an incompatible definition and cannot be accessed}'
                node=WIDGET_TREE(varNode,BITMAP=bitmaps[*,*,*,6],VALUE=varText,SENSITIVE=0,$
                                 UVALUE=STRTRIM(f,2)+varNames[g])
              endif else begin
                oSaveFiles[f]->RESTORE,varNames[g]
                HEAP_FREE,TEMPORARY((SCOPE_VARFETCH(varNames[g],LEVEL=0)))
                ;re-create IDL_SAVEFILE object so heap variable data can be accessed in the future:
                OBJ_DESTROY,oSaveFiles[f]
                oSaveFiles[f]=OBJ_NEW('IDL_SAVEFILE',files[f],/RELAXED_STRUCTURE_ASSIGNMENT)
                node=WIDGET_TREE(varNode,BITMAP=bitmaps[*,*,*,6],VALUE=varText,$
                                 UVALUE=STRTRIM(f,2)+varNames[g])
                dataTreeIDs=[dataTreeIDs,node]
              endelse
            end
          else : $
            begin ;normal:
              if (varNDims EQ 0) then $
                node=WIDGET_TREE(varNode,BITMAP=bitmaps[*,*,*,0],VALUE=varText,UVALUE=STRTRIM(f,2)+varNames[g]) $
              else $
                node=WIDGET_TREE(varNode,BITMAP=bitmaps[*,*,*,3],VALUE=varText,UVALUE=STRTRIM(f,2)+varNames[g])
              dataTreeIDs=[dataTreeIDs,node]
            end
        endcase
      endfor
    endif
    if (saveContents[f].n_sysvar NE 0) then begin
      systemNode=WIDGET_TREE(saveNode,/FOLDER,VALUE='SYSTEM VARIABLES')
      systemNames=oSaveFiles[f]->NAMES(/SYSTEM_VARIABLE,COUNT=count)
      for g=0,count-1 do begin
        node=WIDGET_TREE(systemNode,BITMAP=bitmaps[*,*,*,9],VALUE=systemNames[g])
      endfor
    endif
    if (saveContents[f].n_common NE 0) then begin
      commonNode=WIDGET_TREE(saveNode,/FOLDER,VALUE='COMMON BLOCK DEFINITIONS')
      commonNames=oSaveFiles[f]->NAMES(/COMMON_BLOCK,COUNT=count)
      for g=0,count-1 do begin
        node=WIDGET_TREE(commonNode,BITMAP=bitmaps[*,*,*,7],VALUE=commonNames[g],/FOLDER)
        commonVarNames=oSaveFiles[f]->NAMES(COMMON_VARIABLE=commonNames[g],COUNT=varCount)
        if (varCount GT 0) then begin
          for h=0,varCount-1 do begin
            varNode=WIDGET_TREE(node,BITMAP=bitmaps[*,*,*,8],VALUE=commonVarNames[h])
          endfor
        endif
      endfor
    endif
    if (saveContents[f].n_sysvar NE 0) then begin
      structureNode=WIDGET_TREE(saveNode,/FOLDER,VALUE='NAMED STRUCTURE DEFINITIONS')
      structureNames=oSaveFiles[f]->NAMES(/SYSTEM_VARIABLE,COUNT=count)
      for g=0,count-1 do begin
        node=WIDGET_TREE(structureNode,BITMAP=bitmaps[*,*,*,5],VALUE=structureNames[g])
      endfor
    endif
  endelse
  ;check to see if a structure containing a pointer was restored and its heap
  ;subsequently destroyed; if so, must re-create a new save file object for this file:
  if (structRestoreFlag EQ 1) then begin
    HEAP_FREE,TEMPORARY(save_browser_structure)
    OBJ_DESTROY,oSaveFiles[f]
    oSaveFiles[f]=OBJ_NEW('IDL_SAVEFILE',files[f],/RELAXED_STRUCTURE_ASSIGNMENT)
  endif
endfor
RETURN,tree
END
;****************************************************************************************************


;****************************************************************************************************
;+
; <PRO SAVE_BROWSER_OPEN>
; General description of routine
;
; @Param
;   event {in|out} {optional|required} {type=} {default=}
;     No Description
;
; @Author
;   Adam O'Connor
;
; @Copyright
;   Copyright &#169; 2006 ITT Industries. All rights reserved.
;
; @Hidden
;-
PRO SAVE_BROWSER_OPEN,event
COMPILE_OPT idl2,hidden
CD,CURRENT=currentDir
;error handling ... reset !ERROR_STATE:
MESSAGE,/RESET
;catch error:
CATCH,error
if (error NE 0) then begin ;error caught:
  if (!ERROR_STATE.NAME EQ 'IDL_M_SR_BADSAVFIL') then begin
    badFilesFlag=1B
    ;if only 1 file remains in list then display message and terminate execution of this program:
    if (N_ELEMENTS(files) EQ 1) then begin
      void=DIALOG_MESSAGE('None of the selected files appear to be valid IDL save files.')
      CD,currentDir
      RETURN
    endif else begin ;remove the offending file from the list and continue:
      ;destroy any existing objects that may have been created:
      for j=0,N_ELEMENTS(oSaveFiles)-1 do begin
        if OBJ_VALID(oSaveFiles[j]) then OBJ_DESTROY,oSaveFiles[j]
      endfor
      if (SIZE(badFiles,/TYPE) EQ 0) then badFiles=[files[i]] else badFiles=[badFiles,files[i]]
      files=files[WHERE(files NE files[i])]
      GOTO,tryAgain
    endelse
  endif else begin
    ;obtain last message:
    HELP,/LAST_MESSAGE,OUTPUT=traceback
    messageStr=['Error Caught :','',traceback]
    ;display in dialog:
    void=DIALOG_MESSAGE(messageStr,/ERROR)
    if (SIZE(oSaveFiles,/TYPE) NE 0) then begin
      for j=0,N_ELEMENTS(oSaveFiles)-1 do begin
        if OBJ_VALID(oSaveFiles[j]) then OBJ_DESTROY,oSaveFiles[j]
      endfor
    endif
    CD,currentDir
    RETURN
  endelse
endif
;obtain state structure for top-level-base from its uvalue:
WIDGET_CONTROL,event.top,GET_UVALUE=pState
;prompt user to select *.sav files:
start:
files=DIALOG_PICKFILE(TITLE='Select IDL save file(s) to browse',/MULTIPLE,/MUST_EXIST,GET_PATH=path,$
                      FILTER=[['*.sav;*.SAV;*.dat;*.DAT','*.*'],$
                      ['IDL Save File (*.sav; *.dat)','All Files (*.*)']])
WAIT,0.1
;if user hit "Cancel" then terminate execution of this program:
if (files[0] EQ '') then begin
  CD,currentDir
  RETURN
endif
files=files[SORT(files)]
files=files[UNIQ(files)]
CD,path
nFiles=N_ELEMENTS(files)
;no more than 25 files can be opened at a time because the savefile object consumes LUNs:
if (nFiles GT 25) then begin
  void=DIALOG_MESSAGE(['You have selected '+STRTRIM(nFiles,2)+' save files to browse.  The maximum',$
                       'number of save files you can browse at one time using this',$
                       'utility is 25.  Please select a smaller number of save files.'])
  GOTO,start
endif
WIDGET_CONTROL,/HOURGLASS
;workaround CR# 17797 (Win: DIALOG_PICKFILE adds an extra "\" to the path for multiple
;                      files selected from root directory):
if (STRUPCASE(!VERSION.OS_FAMILY) EQ 'WINDOWS') then begin
  doubleSep=PATH_SEP()+PATH_SEP()
  if (MAX(STRPOS(files,doubleSep,2)) NE -1) then begin
    for i=0,nFiles-1 do begin
      files[i]=STRMID(files[i],0,STRPOS(files[i],doubleSep))+STRMID(files[i],STRPOS(files[i],doubleSep)+1)
    endfor
  endif
endif
;perform preliminary test to see if selected files are valid IDL save files based on first 2 bytes:
header=BYTARR(2)
goodFiles=MAKE_ARRAY(nFiles,/BYTE,VALUE=1B)
cannotAccess=MAKE_ARRAY(nFiles,/BYTE,VALUE=0B)
for i=0,nFiles-1 do begin
  ;first, make sure the file is accessible and at least 4 bytes in length:
  if ((FILE_INFO(files[i])).exists EQ 0) or ((FILE_INFO(files[i])).read EQ 0) then begin
    goodFiles[i]=0B
    cannotAccess[i]=1B
    CONTINUE
  endif
  if ((FILE_INFO(files[i])).size LT 4) then begin
    goodFiles[i]=0B
    CONTINUE
  endif
  OPENR,lun,files[i],/GET
  READU,lun,header
  FREE_LUN,lun
  ;the first two bytes of all IDL save files are the letters "S" (for "Save") followed
  ;by "R" (for "Restore"):
  if (STRUPCASE(STRING(header)) NE 'SR') then goodFiles[i]=0B
endfor
badIndex=WHERE(cannotAccess EQ 1,badCount)
if (badCount NE 0) then begin
  void=DIALOG_MESSAGE(['Unable to access and read from the following files :','',files[badIndex]])
  if (badCount EQ nFiles) then begin
    !QUIET=quietInit
    SET_PLOT,currDevice
    CD,currentDir
    RETURN
  endif
endif
goodIndex=WHERE(goodFiles EQ 1,goodCount)
if (goodCount EQ 0) then begin
  void=DIALOG_MESSAGE('None of the selected files appear to be valid IDL save files.')
  CD,currentDir
  RETURN
endif
if (goodCount NE nFiles) then begin
  badIndex=WHERE(goodFiles EQ 0)
  void=DIALOG_MESSAGE(['The following files do not appear to be valid IDL save files :','',files[badIndex]],/INFO)
endif
;throw away the files that are definitely not IDL save files; remaining files *may* be valid
;IDL save files, but not necessarily since other files may start with the same byte sequence:
files=files[goodIndex]
;cleanup the existing save file objects:
for s=0,N_ELEMENTS(*(*pState).oSaveFiles)-1 do begin
  if OBJ_VALID((*(*pState).oSaveFiles)[s]) then OBJ_DESTROY,(*(*pState).oSaveFiles)[s]
endfor
PTR_FREE,(*pState).oSaveFiles
PTR_FREE,(*pState).dataTreeIDs
badFilesFlag=0B
tryAgain:
;attempt to create save file object with selected file(s); if an error occurs creating the IDL_SAVEFILE
;object it is caught above and dealt with accordingly:
nFiles=N_ELEMENTS(files)
oSaveFiles=OBJARR(nFiles)
i=0
oSaveFiles[0]=OBJ_NEW('IDL_SAVEFILE',files[i],/RELAXED_STRUCTURE_ASSIGNMENT)
saveContents=oSaveFiles[0]->CONTENTS()
if (nFiles GT 1) then begin
  for i=1,nFiles-1 do begin
    oSaveFiles[i]=OBJ_NEW('IDL_SAVEFILE',files[i],/RELAXED_STRUCTURE_ASSIGNMENT)
    currContents=oSaveFiles[i]->CONTENTS()
    saveContents=[saveContents,TEMPORARY(currContents)]
  endfor
endif
if (badFilesFlag EQ 1) then begin
  void=DIALOG_MESSAGE(['The following files do not appear to be valid IDL save files :','',badFiles],/INFO)
endif
;make sure user has write permissions to the folder where the selected file(s) are located:
OPENW,lun,'void.tmp',/GET,ERROR=writeError
if (writeError NE 0) then begin ;unable to write to the current working directory:
  void=DIALOG_MESSAGE(['Unable to write to the location of the selected save file(s) :','',path,'',$
                       'The save file(s) must be located in a directory with open write',$
                       'permissions for the current user before data variables can be',$
                       'output using this program.  Consequently, the ability to save',$
                       'data variables will be disabled.'])
  (*pState).saveFlag=0B
endif else begin
  FREE_LUN,lun
  FILE_DELETE,'void.tmp',/QUIET
  (*pState).saveFlag=1B
endelse
saveID=WIDGET_INFO(event.top,FIND_BY_UNAME='SAVE')
WIDGET_CONTROL,saveID,SENSITIVE=0
outputID=WIDGET_INFO(event.top,FIND_BY_UNAME='OUTPUT')
WIDGET_CONTROL,outputID,SENSITIVE=(*pState).saveFlag
;create new tree widget:
WIDGET_CONTROL,(*pState).tree,/DESTROY
tree=SAVE_BROWSER_CREATE_TREE(event.top,(*pState).treeBase,files,oSaveFiles,saveContents,$
                                       GET_DATATREEIDS=dataTreeIDs,RESET_SESSION_WARN=resetSessionWarnFlag)
if (resetSessionWarnFlag EQ 1) then (*pState).resetSessionWarnFlag=1B
if (tree EQ -1) then begin
  WIDGET_CONTROL,event.top,/DESTROY
  for j=0,N_ELEMENTS(oSaveFiles)-1 do begin
    if OBJ_VALID(oSaveFiles[j]) then OBJ_DESTROY,oSaveFiles[j]
  endfor
  RETURN
endif
(*pState).tree=tree
(*pState).dataTreeIDs=PTR_NEW(dataTreeIDs)
(*pState).oSaveFiles=PTR_NEW(oSaveFiles)
(*pState).files=files
END
;****************************************************************************************************


;****************************************************************************************************
;+
; <PRO SAVE_BROWSER>
;   Provides a graphical user interface (GUI) to examine IDL save files, restore
;   selected data variables, and export data variables to a variety of alternate
;   output format files on disk.  The extraction of IDL object instance data is not
;   supported by this program. This program also does not support the output of data
;   that is stored in heap variables referenced by nested pointers (i.e. pointers to
;   pointers).  In these scenarios the user must restore the save file(s) into a
;   standard licensed copy of the IDL software package in order to access any data
;   that these variables may reference.  Finally, the tree widget folder expansion
;   of nested structures is limited to 2 levels.<BR>
;   <BR>
;   If a save file is selected that contains routines it will be restored into the
;   current IDL session in an effort to determine the arguments and keywords for
;   these routines.  If this occurs within an interactive session of IDL the user
;   will be prompted with an informative message dialog upon termination of the program.<BR>
;   <BR>
;   User-Contributed Library webpage :
;   <A HREF="http://www.rsinc.com/codebank/search.asp?FID=275">http://www.rsinc.com/codebank/search.asp?FID=275</A>
;
; @Author
;   Adam O'Connor<BR>
;   <A HREF="mailto:adam_e_oconnor\@yahoo.com">&lt;adam_e_oconnor\@yahoo.com&gt;</A>
;
; @Copyright
;   Copyright &#169; 2006 ITT Industries. All rights reserved.
;
; @Categories
;   Save, SAV, Browser, Access, Widget
;
; @Examples
;   IDL> save_browser
;
; @History
;   Created: January, 2004 <BR>
;   Improved memory management: February, 2004 <BR>
;   Added support for display sizes smaller than 1024 x 768 and modified output
;     functionality so that variables with same name in different save files can
;     be output at the same time without filename conflicts: July, 2004 <BR>
;   Added informative message dialog upon completion if save file(s) containing
;     routines were restored into the current IDL session: August, 2004 <BR>
;   Fixed problem with output of negative floating-point values to ASCII: January, 2005 <BR>
;
; @Requires
;   6.1
;
; @Restrictions
;   Although this program was generated by an ITT employee, it is in no way supported,
;   maintained, warranted, or guaranteed by ITT Industries.  Consequently, the ITT
;   Technical Support Group is not obligated to address questions related to this
;   program.  Use this program at your own risk.
;-
PRO SAVE_BROWSER
COMPILE_OPT idl2
;suppress informational messaging:
quietInit=!QUIET
!QUIET=1
currDevice=!D.NAME
CD,CURRENT=currentDir
;error handling ... reset !ERROR_STATE:
MESSAGE,/RESET
;catch error:
CATCH,error
if (error NE 0) then begin ;error caught:
  if (!ERROR_STATE.NAME EQ 'IDL_M_SR_BADSAVFIL') then begin
    badFilesFlag=1B
    ;if only 1 file remains in list then display message and terminate execution of this program:
    if (N_ELEMENTS(files) EQ 1) then begin
      void=DIALOG_MESSAGE('None of the selected files appear to be valid IDL save files.')
      !QUIET=quietInit
      SET_PLOT,currDevice
      CD,currentDir
      RETURN
    endif else begin ;remove the offending file from the list and continue:
      ;destroy any existing objects that may have been created:
      for j=0,N_ELEMENTS(oSaveFiles)-1 do begin
        if OBJ_VALID(oSaveFiles[j]) then OBJ_DESTROY,oSaveFiles[j]
      endfor
      if (SIZE(badFiles,/TYPE) EQ 0) then badFiles=[files[i]] else badFiles=[badFiles,files[i]]
      files=files[WHERE(files NE files[i])]
      GOTO,tryAgain
    endelse
  endif else begin
    ;obtain last message:
    HELP,/LAST_MESSAGE,OUTPUT=traceback
    messageStr=['Error Caught :','',traceback]
    ;display in dialog:
    void=DIALOG_MESSAGE(messageStr,/ERROR)
    if (SIZE(oSaveFiles,/TYPE) NE 0) then begin
      for j=0,N_ELEMENTS(oSaveFiles)-1 do begin
        if OBJ_VALID(oSaveFiles[j]) then OBJ_DESTROY,oSaveFiles[j]
      endfor
    endif
    !QUIET=quietInit
    SET_PLOT,currDevice
    CD,currentDir
    RETURN
  endelse
endif
;ignore beta and development build versions of IDL because string to float conversion will fail:
betaTest=STRPOS(STRLOWCASE(!VERSION.RELEASE),'beta')
buildTest=STRPOS(STRLOWCASE(!VERSION.RELEASE),'build')
;check to make sure the version of IDL running is 6.1 or newer:
if (betaTest EQ -1) and (buildTest EQ -1) then begin
  if (FLOAT(!VERSION.RELEASE) LT 6.1) then begin
    void=DIALOG_MESSAGE('SAVE_BROWSER is only supported in IDL v.6.1 or newer.')
    !QUIET=quietInit
    SET_PLOT,currDevice
    CD,currentDir
    RETURN
  endif
endif
;make sure active graphics device is appropriate:
if (STRUPCASE(!VERSION.OS_FAMILY) EQ 'WINDOWS') then $
  SET_PLOT,'WIN' $
else $
  SET_PLOT,'X'
;determine screen size:
DEVICE,GET_SCREEN_SIZE=screenSize
screenSize=LONG(screenSize)
xCenter=screenSize[0]/2
yCenter=screenSize[1]/2
;prompt user to select *.sav files:
start:
files=DIALOG_PICKFILE(TITLE='Select IDL save file(s) to browse',/MULTIPLE,/MUST_EXIST,GET_PATH=path,$
                      FILTER=[['*.sav;*.SAV;*.dat;*.DAT','*.*'],$
                      ['IDL Save File (*.sav; *.dat)','All Files (*.*)']])
WAIT,0.1
;if user hit "Cancel" then terminate execution of this program:
if (files[0] EQ '') then begin
  !QUIET=quietInit
  SET_PLOT,currDevice
  CD,currentDir
  RETURN
endif
files=files[SORT(files)]
files=files[UNIQ(files)]
CD,path
nFiles=N_ELEMENTS(files)
;no more than 25 files can be opened at a time because the savefile object consumes LUNs:
if (nFiles GT 25) then begin
  void=DIALOG_MESSAGE(['You have selected '+STRTRIM(nFiles,2)+' save files to browse.  The maximum',$
                       'number of save files you can browse at one time using this',$
                       'utility is 25.  Please select a smaller number of save files.'])
  GOTO,start
endif
;workaround CR# 17797 (Win: DIALOG_PICKFILE adds an extra "\" to the path for multiple
;                      files selected from root directory):
if (STRUPCASE(!VERSION.OS_FAMILY) EQ 'WINDOWS') then begin
  doubleSep=PATH_SEP()+PATH_SEP()
  if (MAX(STRPOS(files,doubleSep,2)) NE -1) then begin
    for i=0,nFiles-1 do begin
      files[i]=STRMID(files[i],0,STRPOS(files[i],doubleSep))+STRMID(files[i],STRPOS(files[i],doubleSep)+1)
    endfor
  endif
endif
;perform preliminary test to see if selected files are valid IDL save files based on first 2 bytes:
header=BYTARR(2)
goodFiles=MAKE_ARRAY(nFiles,/BYTE,VALUE=1B)
cannotAccess=MAKE_ARRAY(nFiles,/BYTE,VALUE=0B)
for i=0,nFiles-1 do begin
  ;first, make sure the file is accessible and at least 4 bytes in length:
  if ((FILE_INFO(files[i])).exists EQ 0) or ((FILE_INFO(files[i])).read EQ 0) then begin
    goodFiles[i]=0B
    cannotAccess[i]=1B
    CONTINUE
  endif
  if ((FILE_INFO(files[i])).size LT 4) then begin
    goodFiles[i]=0B
    CONTINUE
  endif
  OPENR,lun,files[i],/GET
  READU,lun,header
  FREE_LUN,lun
  ;the first two bytes of all IDL save files are the letters "S" (for "Save") followed
  ;by "R" (for "Restore"):
  if (STRUPCASE(STRING(header)) NE 'SR') then goodFiles[i]=0B
endfor
badIndex=WHERE(cannotAccess EQ 1,badCount)
if (badCount NE 0) then begin
  void=DIALOG_MESSAGE(['Unable to access and read from the following files :','',files[badIndex]])
  if (badCount EQ nFiles) then begin
    !QUIET=quietInit
    SET_PLOT,currDevice
    CD,currentDir
    RETURN
  endif
endif
goodIndex=WHERE(goodFiles EQ 1,goodCount)
if (goodCount EQ 0) then begin
  void=DIALOG_MESSAGE('None of the selected files appear to be valid IDL save files.')
  !QUIET=quietInit
  SET_PLOT,currDevice
  CD,currentDir
  RETURN
endif
if (goodCount NE nFiles) then begin
  badIndex=WHERE(goodFiles EQ 0)
  void=DIALOG_MESSAGE(['The following files do not appear to be valid IDL save files :','',files[badIndex]],/INFO)
endif
;throw away the files that are definitely not IDL save files; remaining files *may* be valid
;IDL save files, but not necessarily since other files may start with the same byte sequence:
files=files[goodIndex]
badFilesFlag=0B
tryAgain:
;attempt to create save file object with selected file(s); if an error occurs creating the IDL_SAVEFILE
;object it is caught above and dealt with accordingly:
nFiles=N_ELEMENTS(files)
oSaveFiles=OBJARR(nFiles)
i=0
oSaveFiles[0]=OBJ_NEW('IDL_SAVEFILE',files[i],/RELAXED_STRUCTURE_ASSIGNMENT)
saveContents=oSaveFiles[0]->CONTENTS()
if (nFiles GT 1) then begin
  for i=1,nFiles-1 do begin
    oSaveFiles[i]=OBJ_NEW('IDL_SAVEFILE',files[i],/RELAXED_STRUCTURE_ASSIGNMENT)
    currContents=oSaveFiles[i]->CONTENTS()
    saveContents=[saveContents,TEMPORARY(currContents)]
  endfor
endif
if (badFilesFlag EQ 1) then begin
  void=DIALOG_MESSAGE(['The following files do not appear to be valid IDL save files :','',badFiles],/INFO)
endif
;make sure user has write permissions to the folder where the selected file(s) are located:
OPENW,lun,'void.tmp',/GET,ERROR=writeError
if (writeError NE 0) then begin ;unable to write to the current working directory:
  void=DIALOG_MESSAGE(['Unable to write to the location of the selected save file(s) :','',path,'',$
                       'The save file(s) must be located in a directory with open write',$
                       'permissions for the current user before data variables can be',$
                       'output using this program.  Consequently, the ability to save',$
                       'data variables will be disabled.'])
  saveFlag=0B
endif else begin
  FREE_LUN,lun
  FILE_DELETE,'void.tmp',/QUIET
  saveFlag=1B
endelse
;read-in bitmaps:
bitmaps=BYTARR(16,16,3,11)
;
scalarBMP=BYTARR(16,16,3,/NOZERO)
image8bit=READ_BMP(FILEPATH('new.bmp',SUBDIR=['resource','bitmaps']),r,g,b)
scalarBMP[0,0,0]=r[image8bit]
scalarBMP[0,0,1]=g[image8bit]
scalarBMP[0,0,2]=b[image8bit]
bitmaps[*,*,*,0]=TEMPORARY(scalarBMP)
;
textBMP=BYTARR(16,16,3,/NOZERO)
image8bit=READ_BMP(FILEPATH('text.bmp',SUBDIR=['resource','bitmaps']),r,g,b)
textBMP[0,0,0]=r[image8bit]
textBMP[0,0,1]=g[image8bit]
textBMP[0,0,2]=b[image8bit]
bitmaps[*,*,*,1]=TEMPORARY(textBMP)
;
strarrBMP=BYTARR(16,16,3,/NOZERO)
image8bit=READ_BMP(FILEPATH('ascii.bmp',SUBDIR=['resource','bitmaps']),r,g,b)
strarrBMP[0,0,0]=r[image8bit]
strarrBMP[0,0,1]=g[image8bit]
strarrBMP[0,0,2]=b[image8bit]
bitmaps[*,*,*,2]=TEMPORARY(strarrBMP)
;
arrayBMP=BYTARR(16,16,3,/NOZERO)
image8bit=READ_BMP(FILEPATH('binary.bmp',SUBDIR=['resource','bitmaps']),r,g,b)
arrayBMP[0,0,0]=r[image8bit]
arrayBMP[0,0,1]=g[image8bit]
arrayBMP[0,0,2]=b[image8bit]
bitmaps[*,*,*,3]=TEMPORARY(arrayBMP)
;
pointerBMP=BYTARR(16,16,3,/NOZERO)
image8bit=READ_BMP(FILEPATH('arrow.bmp',SUBDIR=['resource','bitmaps']),r,g,b)
pointerBMP[0,0,0]=r[image8bit]
pointerBMP[0,0,1]=g[image8bit]
pointerBMP[0,0,2]=b[image8bit]
bitmaps[*,*,*,4]=TEMPORARY(pointerBMP)
;
structureBMP=BYTARR(16,16,3,/NOZERO)
image8bit=READ_BMP(FILEPATH('dm.bmp',SUBDIR=['resource','bitmaps']),r,g,b)
structureBMP[0,0,0]=r[image8bit]
structureBMP[0,0,1]=g[image8bit]
structureBMP[0,0,2]=b[image8bit]
bitmaps[*,*,*,5]=TEMPORARY(structureBMP)
;
objectBMP=BYTARR(16,16,3,/NOZERO)
image8bit=READ_BMP(FILEPATH('mcr.bmp',SUBDIR=['resource','bitmaps']),r,g,b)
objectBMP[0,0,0]=r[image8bit]
objectBMP[0,0,1]=g[image8bit]
objectBMP[0,0,2]=b[image8bit]
bitmaps[*,*,*,6]=TEMPORARY(objectBMP)
;
commonBMP=BYTARR(16,16,3,/NOZERO)
image8bit=READ_BMP(FILEPATH('dataspace.bmp',SUBDIR=['resource','bitmaps']),r,g,b)
commonBMP[0,0,0]=r[image8bit]
commonBMP[0,0,1]=g[image8bit]
commonBMP[0,0,2]=b[image8bit]
bitmaps[*,*,*,7]=TEMPORARY(commonBMP)
;
commonVarBMP=BYTARR(16,16,3,/NOZERO)
image8bit=READ_BMP(FILEPATH('view.bmp',SUBDIR=['resource','bitmaps']),r,g,b)
commonVarBMP[0,0,0]=r[image8bit]
commonVarBMP[0,0,1]=g[image8bit]
commonVarBMP[0,0,2]=b[image8bit]
bitmaps[*,*,*,8]=TEMPORARY(commonVarBMP)
;
systemBMP=BYTARR(16,16,3,/NOZERO)
image8bit=READ_BMP(FILEPATH('commandline.bmp',SUBDIR=['resource','bitmaps']),r,g,b)
systemBMP[0,0,0]=r[image8bit]
systemBMP[0,0,1]=g[image8bit]
systemBMP[0,0,2]=b[image8bit]
bitmaps[*,*,*,9]=TEMPORARY(systemBMP)
;
routineBMP=BYTARR(16,16,3,/NOZERO)
image8bit=READ_BMP(FILEPATH('importd.bmp',SUBDIR=['resource','bitmaps']),r,g,b)
routineBMP[0,0,0]=r[image8bit]
routineBMP[0,0,1]=g[image8bit]
routineBMP[0,0,2]=b[image8bit]
bitmaps[*,*,*,10]=TEMPORARY(routineBMP)
;
;create GUI:
tlb=WIDGET_BASE(TITLE='IDL Save File Browser',/COLUMN,/TLB_SIZE_EVENTS,MAP=0)
  treeBase=WIDGET_BASE(tlb,/FRAME)
  tree=SAVE_BROWSER_CREATE_TREE(tlb,treeBase,files,oSaveFiles,saveContents,$
       GET_DATATREEIDS=dataTreeIDs,BITMAPS=bitmaps,SCREEN_SIZE=screenSize,$
       RESET_SESSION_WARN=resetSessionWarnFlag)
  if (tree EQ -1) then begin
    !QUIET=quietInit
    SET_PLOT,currDevice
    CD,currentDir
    for j=0,N_ELEMENTS(oSaveFiles)-1 do begin
      if OBJ_VALID(oSaveFiles[j]) then OBJ_DESTROY,oSaveFiles[j]
    endfor
    RETURN
  endif
  formatBase=WIDGET_BASE(tlb,/ROW,/FRAME,/ALIGN_CENTER,SENSITIVE=saveFlag,UNAME='OUTPUT')
    formatLabel=WIDGET_LABEL(formatBase,VALUE='Output Format =  ',/ALIGN_CENTER)
    outputBase=WIDGET_BASE(formatBase,/ROW,/NONEXCLUSIVE)
      spaceBttn=WIDGET_BUTTON(outputBase,VALUE='Space Delimited ASCII (*.txt)',UNAME='SPACE',UVALUE='CHECKBOX')
      tabBttn=WIDGET_BUTTON(outputBase,VALUE='Tab Delimited ASCII (*.tab)',UNAME='TAB',UVALUE='CHECKBOX')
      commaBttn=WIDGET_BUTTON(outputBase,VALUE='Comma Separated ASCII (*.csv)',UNAME='COMMA',UVALUE='CHECKBOX')
      binaryBttn=WIDGET_BUTTON(outputBase,VALUE='Structured Binary (*.bin)',UNAME='BINARY',UVALUE='CHECKBOX')
  optionsBase=WIDGET_BASE(tlb,/FRAME,/ROW,/ALIGN_CENTER)
    restoreFlag=0B
    HELP,CALLS=callStack
    interactiveIDL=MAX(STRPOS(STRUPCASE(callStack),'.PRO'))
    if ~(LMGR(/VM)) and ~(LMGR(/RUNTIME)) and (interactiveIDL NE -1) then begin
      restoreBttn=WIDGET_BUTTON(optionsBase,VALUE=' Restore Selected Data Variables Into Current Session ',$
                                UVALUE='RESTORE',UNAME='RESTORE',EVENT_PRO='SAVE_BROWSER_RESTORE',$
                                /FRAME,SENSITIVE=0)
      spacer=WIDGET_LABEL(optionsBase,VALUE=' ')
      restoreFlag=1B
    endif
    saveBttn=WIDGET_BUTTON(optionsBase,/FRAME,VALUE=' Output Selected Data Variables ',SENSITIVE=0,$
                           UVALUE='SAVE',UNAME='SAVE',EVENT_PRO='SAVE_BROWSER_SAVE')
    spacer=WIDGET_LABEL(optionsBase,VALUE=' ')
    openBttn=WIDGET_BUTTON(optionsBase,/FRAME,VALUE=' Open New Save Files ',UVALUE='OPEN',$
                           EVENT_PRO='SAVE_BROWSER_OPEN')
    spacer=WIDGET_LABEL(optionsBase,VALUE=' ')
    cancelBttn=WIDGET_BUTTON(optionsBase,/FRAME,VALUE=' Cancel ',UVALUE='CANCEL',$
                             EVENT_PRO='SAVE_BROWSER_CANCEL')
;realize the widget:
WIDGET_CONTROL,tlb,/REALIZE
;obtain the top-level base geometry:
tlbGeom=WIDGET_INFO(tlb,/GEOMETRY)
tlbWidth=tlbGeom.xsize
tlbHeight=tlbGeom.ysize
if (tlbWidth EQ 0) or (tlbHeight EQ 0) then begin
  WIDGET_CONTROL,tlb,TLB_GET_SIZE=tlbSize
  tlbWidth=tlbSize[0]
  tlbHeight=tlbSize[1]
endif
xOffset=xCenter-(tlbGeom.Scr_XSize/2)
WIDGET_CONTROL,tlb,XOFFSET=xOffset,YOFFSET=0
if (saveFlag EQ 1) then WIDGET_CONTROL,spaceBttn,/SET_BUTTON
;display the GUI to the screen:
WIDGET_CONTROL,tlb,/MAP
;create state structure to store information needed by the other event handling procedures:
pState=PTR_NEW({oSaveFiles:PTR_NEW(oSaveFiles),$
                dataTreeIDs:PTR_NEW(dataTreeIDs),$
                files:files,$
                tlbWidth:tlbWidth,$
                currentDir:currentDir,$
                tlbHeight:tlbHeight,$
                xOffset:xOffset,$
                quietInit:quietInit,$
                currDevice:currDevice,$
                treeBase:treeBase,$
                tree:tree,$
                saveFlag:saveFlag,$
                screenSize:screenSize,$
                xCenter:xCenter,$
                yCenter:yCenter,$
                bitmaps:bitmaps,$
                restoreFlag:restoreFlag,$
                resetSessionWarnFlag:resetSessionWarnFlag})
;store this state structure in the uvalue of the top-level-base
;so it can be obtained by other program units:
WIDGET_CONTROL,tlb,SET_UVALUE=pState
;register the GUI with the XMANAGER event handler routine:
XMANAGER,'save_browser',tlb,CLEANUP='save_browser_cleanup',/NO_BLOCK
END
;****************************************************************************************************
