;+
; Logging functionality
;
; @author Rob Redmon
; @copyright NGDC, 2007
; @version 1.0.0
;-


;+
; Logging DEBUG
;
; @param message {in}{required}{type=string value} Message to log.
;-
pro log_debug, message
    check_init
    if ( !LOG.VERBOSITY ge !LOG.LEVEL.DEBUG ) then print_log_message, 'DEBUG', message
end

;+
; Logging INFO
;
; @param message {in}{required}{type=string value} Message to log.
;-
pro log_info, message
    check_init
    if ( !LOG.VERBOSITY ge !LOG.LEVEL.INFO ) then print_log_message, 'INFO', message
end

;+
; Logging WARN
;
; @param message {in}{required}{type=string value} Message to log.
;-
pro log_warn, message
    check_init
    if ( !LOG.VERBOSITY ge !LOG.LEVEL.WARN ) then print_log_message, 'WARN', message
end

;+
; Logging ERROR
;
; @param message {in}{required}{type=string value} Message to log.
;-
pro log_error, message
    check_init
    if ( !LOG.VERBOSITY ge !LOG.LEVEL.ERROR ) then print_log_message, 'ERROR', message
end

;+
; Logging FATAL
;
; @param message {in}{required}{type=string value} Message to log.
;-
pro log_fatal, message
    check_init
    if ( !LOG.VERBOSITY ge !LOG.LEVEL.FATAL ) then print_log_message, 'FATAL', message
end

;+
; Utility for the above functions.  Not for external usage.
;
; @param message {in}{required}{type=string value} Message to log.
;-
pro print_log_message, level, message
    
    log_line = strmid(level,0,1) + ", [" + systime() + "]  " + level + " --: " +  message
    
    if ( !LOG.FILE ne '' ) then begin
        OpenW, lun, !LOG.FILE, /APPEND, /GET_LUN
        printf, lun, log_line
        Free_Lun, lun
    endif $
    else begin
        print, log_line
    endelse
end


;+
; Locally used function.  Not for external usage.  Calls default setup in event not already invoked by external caller.
;
; @param message {in}{required}{type=string value} Message to log.
;-
PRO check_init
    defsysv, '!LOG', EXISTS=e
    if ( e eq 0 ) then logger
end


;+
; Inits Logging functionality
;
; @author Rob Redmon
; @copyright NGDC, 2007
; @keyword VERBOSE  {in}{optional}{type=integer value} 0=none, 1=Fatal, 2=Error, 3=Warn, 4=Info, 5=Debug
; @keyword LOG_FILE {in}{optional}{type=string value} File to write log to.  Empty string = STDOUT.
;-
PRO logger, VERBOSITY=VERBOSITY, LOG_FILE=LOG_FILE

    ; New instance?
    defsysv, '!LOG', EXISTS=e
    if ( ~ e ) then begin
        log_level = create_struct( 'DEBUG', 5, $
                                   'INFO',  4, $
                                   'WARN',  3, $
                                   'ERROR', 2, $
                                   'FATAL', 1, $
                                   'NONE',  0 )

        log = create_struct( 'LEVEL',     log_level, $
                             'FILE',      '', $
                             'VERBOSITY', log_level.INFO )

        defsysv, '!LOG', log
        
    endif
    
    ; Update details
    if ( n_elements( VERBOSITY ) ) then !LOG.VERBOSITY = VERBOSITY
    if ( n_elements( LOG_FILE )  ) then !LOG.FILE      = LOG_FILE

end
