;+
; Given a date-time array in ISO-8601, plot availability by hour
;-

;+
; load a color table
;
; @author Rob Redmon
; @copyright NGDC, 2008
; @keyword DEBUG {in}{optional}{type=boolean} 0 => nothing; ~0 => display color table graphically.
;-
pro load_red_blue_ct, DEBUG=debug

    min_color = 80
    r = fix( [ 0 * intarr( 128 ), min_color + (( 255. - min_color )/( 127. )) * indgen( 128 ) ] )
    g = 0 * intarr( 255 )
    b = reverse( r )

    ; for the grid background
    r[0] = 0
    b[0] = 0

    DEVICE, DECOMPOSED=0
    tvlct, r, g, b

    if ( keyword_set( DEBUG ) ) then TVSCL, DIST(400)
    
end


;+
; Given a date-time array in ISO-8601, plot availability by hour
;
; @author Rob Redmon
; @copyright NGDC, 2007
; @param   date_time_8601 {in}{required}{type=array value} E.g. ["2007-10-01 12:30:15"].
; @keyword TITLE          {in}{optional}{type=string value} E.g. ["Plot of crap"].
; @keyword XTICKS         {in}{optional}{type=string value} E.g. ["Plot of crap"]. 
;-
pro plot_data_available, date_time_8601, TITLE=title, XTICKS=xticks

    dt_arr = time_8601_to_array( date_time_8601 )
    
    year   = dt_arr[0,*]
    month  = dt_arr[1,*]
    day    = dt_arr[2,*]
    hour   = dt_arr[3,*]
    minute = dt_arr[4,*]
    second = dt_arr[5,*]

    jday = julday( month, day, year, hour, minute, second )
    
    ; get color table
    load_red_blue_ct

    ; time axis setup
    date_label=label_date(DATE_FORMAT=['%M/%Y'])

    if ( ~ n_elements( TITLE ) ) then title = ask(' Title?','' )
    if ( ~ n_elements( XTICKS ) ) then xticks = fix( ask(' Number of XTicks?','6' ) )
    plot, jday, hour, position=[0.05,0.15,.95,.95], XTicks=xticks, XTickFormat=['LABEL_DATE'], XTickUnits=['Time'], yrange=[-1,24], ystyle=1, xstyle=1, title=title, xtitle='Date', ytitle='UT hour', psym='6', charsize=1.25, color=0, background=255
    oplot, jday, hour, psym='6', color=50

end
