;+
; Function to scroll the file pointer down to the next line matching a pattern.
;
; @author Rob Redmon
; @copyright NGDC, 2006, version 1.0
; @param lun {in}{required}{out}{type=LUN value} File logical unit.
; @param pattern {in}{required}{type=string value} Pattern to match.
; @param line {out}{type=string value} The line that matched.
; @param nLines {out}{type=integral value} Number of lines scrolled.
; @keyword VERBOSE {in}{type=optional boolean} print, debugging info
; @keyword RECORD {in}{type=optional boolean} record lines skipped into "BUFFER"
; @keyword BUFFER {out}{type=optional string array} if "RECORD" is set, then record lines skipped into "BUFFER"
; @keyword MATCH_START {in}{type=optional boolean} Match pattern from beginning of line
;
; @change 02Feb2009 - Added Keyword "MATCH_START" to force
;-
FUNCTION scroll_lun_to_pattern, lun, pattern, line, nLines, VERBOSE=VERBOSE, RECORD=RECORD, BUFFER=BUFFER, MATCH_START=MATCH_START
   compile_opt idl2 ; DEFINT32, strictarr
   !warn.obs_routines = 1

   nLines = long(0)
   WHILE (~ EOF(lun)) DO BEGIN
      readf, lun, line

      ; verbose?
      if ( keyword_set( VERBOSE ) ) then print, "read: " + line

      ; keep lines read?
      if ( keyword_set( RECORD ) ) then begin
          if ( nLines eq 0 ) then buffer = line $
          else                    buffer = [ buffer, line ]
      endif
              
      ++nLines

      loc = strpos(line, pattern)

      ; Match from start of line?
      if ( keyword_set( MATCH_START ) ) then begin
          if ( loc eq 0 ) then begin
              return, 1
          endif
      endif $

      ; Match anywhere in line?
      else begin
          if ( loc gt -1 ) then begin
              return, 1
          endif
      endelse

   ENDWHILE

   line = ""
   nLines = -1
   return, 0
end
