;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;+
; NAME:
;    Set_Intersection_Positive_Int
; PURPOSE:
;    Computes intersection of two sets of positive integers.  
;    Must be true sets of POSITIVE integers, input set must NOT have repeated elements!
; DESCRIPTION:
; CATEGORY:
;    Utility
; CALLING SEQUENCE:
;    c = Set_Intersection(a,b)
; INPUTS:
;    a - set a
;    b - set b
; OPTIONAL INPUT PARAMETERS:
; KEYWORD INPUT PARAMETERS:
; OUTPUTS:
; KEYWORD OUTPUT PARAMETERS:
; COMMON BLOCKS:
; SIDE EFFECTS:
; RESTRICTIONS:
; PROCEDURE:
; MODIFICATION HISTORY:
;  2006-01-19 Copied from http://www.dfanning.com/tips/set_operations.html
;-
FUNCTION Set_Intersection_Positive_Int, a, b

   junk = where(a lt 0.0, counta)
   junk = where(b lt 0.0, countb)
   IF (counta gt 0 or countb gt 0) then RETURN, -1

   minab = Min(a, Max=maxa) > Min(b, Max=maxb) ;Only need intersection of ranges
   maxab = maxa < maxb
   
   ; If either set is empty, or their ranges don't intersect: result = NULL.
   
   IF maxab LT minab OR maxab LT 0 THEN RETURN, -1
   r = Where((Histogram(a, Min=minab, Max=maxab) NE 0) AND  $
             (Histogram(b, Min=minab, Max=maxab) NE 0), count)
   
   IF count EQ 0 THEN RETURN, -1 ELSE RETURN, r + minab
END
