;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;+
; NAME:
;    Time_8601_To_Array
; PURPOSE:
;    Converts time string in 8601 format to an array of 6 element time arrays to string.
; VERSION:
;    0.2.0
; DESCRIPTION:
; CATEGORY:
;    Utility
; CALLING SEQUENCE:
;    pro Time_8601_To_Array, time
; INPUTS:
;    time - array of ISO 8601 time/date strings, e.g: ['2005-01-31 12:30:00']
; OPTIONAL INPUT PARAMETERS:
; KEYWORD INPUT PARAMETERS:
; OUTPUTS:
;    time - array of time, e.g. [2005, 1, 31, 12, 30, 0]
; KEYWORD OUTPUT PARAMETERS:
; COMMON BLOCKS:
; SIDE EFFECTS:
; RESTRICTIONS:
; PROCEDURE:
; MODIFICATION HISTORY:
;  2006-02-21  Written by Rob Redmon; version 0.1.0
;  2006-10-05  Modified by Rob to allow the time input to be an array of strings; version 0.2.0
;  2007-10-03  Modified by Rob increase the allowed size of the input data array.
;-

FUNCTION time_8601_to_array, time_input

   strlen_time = strlen(time_input)
   junk = where(strlen_time ne 19, count)
   
   if (count ne 0) then begin
      print, "Each element of input must be of format: '2005-01-31 12:30:00'"
      x = y  ; cause fatal error
   endif

   time_output = intarr(6, n_elements(time_input)) 

   for iTime=ulong64( 0 ), n_elements(time_input)-1 do begin
      tmp_time = time_input[iTime]
      year_idx = strpos(tmp_time, '-')
      year = strmid(tmp_time, 0, year_idx)
   
      month_idx = strpos(tmp_time, "-", year_idx+1)
      month = strmid(tmp_time, year_idx+1, (month_idx-year_idx-1))
   
      day_idx = strpos(tmp_time, " ", month_idx+1)
      day = strmid(tmp_time, month_idx+1, (day_idx-month_idx-1))
   
      hour_idx = strpos(tmp_time, ":", day_idx+1)
      hour = strmid(tmp_time, day_idx+1, (hour_idx-day_idx-1))
   
      minute_idx = strpos(tmp_time, ":", hour_idx+1)
      minute = strmid(tmp_time, hour_idx+1, (minute_idx-hour_idx-1))
   
      second = strmid(tmp_time, minute_idx+1)
   
      time_output[*,iTime] = fix([year, month, day, hour, minute, second])
   
      ;print, "year:", fix(year), ", month:", month, ", day:", day, ", hour:", hour, ", minute:", minute, ", second:", second
   endfor

   ;print, "time:", time_output

   return, time_output

END

PRO test_time_8601_to_array
   time_str_invalid = [['2005-01-31 12:35'], $
               ['2005-01-31 23:59:59']]
   time_str_valid = [['2005-01-31 12:35:59'], $
               ['2005-01-31 23:59:59']]

   print, "Test1: single input"
   print, "   "+time_str_valid[0]+"->",Time_8601_To_Array(time_str_valid[0])

   print, "Test2: array input"
   print, "   "+time_str_valid+"->",Time_8601_To_Array(time_str_valid)

   print, "Test3: invalid input, this one will cause fatal error and you must 'retall' to get back to Main"
   print, "   "+time_str_invalid+"->",Time_8601_To_Array(time_str_invalid)

END
