;+
; Converts array of 6 element time arrays to string.
;
; @author Rob Redmon
; @copyright NGDC, 2007
; @param time {in}{required}{type=array double or float} time array to convert.  e.g. [[2005, 1, 31, 12, 30, 0], [2003, 2, 28, 11, 14]] for "2005-1-31 12:30:0", "2003-2-28 11:14:0"
; @keyword FOR_FILENAME {in}{optional}{type=boolean value} If set, then return strings filename friendly (e.g. no spaces, colons, etc in usual time.
; @returns Array of strings that look SIMILAR to ISO 8601.  Not exactly the same because I'm too lazy to fully implement that standard and I just needed a short and sweet handy utility.
;
; MODIFICATION HISTORY:
;  2005-11-23  Written by Rob Redmon
;  2007-10-25  Fixed "seconds" output.  They were always 0.
;              Rewrote the documentation of this function in more appropriate format.
;-

FUNCTION time_array_to_8601, time, FOR_FILENAME=FOR_FILENAME

   if (n_elements(time[*,0]) ne 6) then x = y  ; cause fatal error

   formatDateTime = '(C(CYI, "-", CMOI2.2, "-", CDI2.2, " ", CHI2.2, ":", CMI2.2, ":", CSI2.2))'
   if ( keyword_set( FOR_FILENAME ) ) then formatDateTime = '(C(CYI, "-", CMOI2.2, "-", CDI2.2, "_", CHI2.2, "", CMI2.2, "", CSI2.2))'

   julTime = julday( time[1,*], time[2,*], time[0,*], time[3,*], time[4,*], time[5,*] )

   return, string(julTime, format = formatDateTime)

END

