;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;+
; NAME:
;    Write_Array_To_CSV
; PURPOSE:
;    Write 2 dimensional N x M array to comma separated value (CSV) file
; DESCRIPTION:
; CATEGORY:
;    Utility
; CALLING SEQUENCE:
;    pro WriteArrayToCSV array_to_write, filename, SEPARATOR=separator, HEADER=header, INDENT=indent
; INPUTS:
;    array_to_write - duh.
;    filename - duh
; OPTIONAL INPUT PARAMETERS:
; KEYWORD INPUT PARAMETERS:
;    HEADER - string array of column names
;    INDENT - string (e.g. "  ") to use for print statements
; OUTPUTS:
; KEYWORD OUTPUT PARAMETERS:
; COMMON BLOCKS:
; SIDE EFFECTS:
; RESTRICTIONS:
; PROCEDURE:
; MODIFICATION HISTORY:
;  2005-11-23  Written by Rob Redmon
;-

PRO Write_Array_To_CSV, data, filename, HEADER=header, SEPARATOR=separator, INDENT=indent

   ; boiler plate
   if (keyword_set(INDENT) ne 1) then INDENT = "   "
   LOCAL_INDENT = INDENT + INDENT

   myName = "EventStats.pro"
   print, INDENT+"=>=>=>=>=>=>=>=>=>=>=>=>=>=>=>=>=>=>=>=>=>=>=>=>=>=>=>=>=>=>=>=>=>=>=>=>"
   print, INDENT + "Starting: "+myName


   if (keyword_set(SEPARATOR) ne 1) then separator = ","

   nCols = n_elements(data[*,0])
   nRows = n_elements(data[0,*])

   print, LOCAL_INDENT+"Writing array to ", filename
   print, LOCAL_INDENT+"Array dimensions:", nCols, " columns   X ", nRows, " rows."

   ; formats
   format = '('+strtrim(n_elements(nCols),2)+'(A, :, "'+separator+'"))'


   help, data

   OpenW, lun_custom, filename, /Get_Lun

   ; header
   row = "" + strtrim(header[0],2)
   if (nCols gt 1) then begin
      for iCol=1, nCols-1 do begin
         row = row + "," + strtrim(header[iCol],2)
      endfor
   endif
   printf, lun_custom, row, FORMAT=format

   ; data
   for iRow=0L, nRows-1 do begin

      row = "" + strtrim(data[0,iRow],2)
      if (nCols gt 1) then begin
         for iCol=1, nCols-1 do begin
            row = row + "," + strtrim(data[iCol,iRow],2)
         endfor
      endif
      printf, lun_custom, row, FORMAT=format

   endfor

   Free_lun, lun_custom

   print, INDENT+"Done: "+myName
   print, INDENT+"<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=<=<="


END
